"""
对比两个db3的主键个数及内容是否相同
以及主键下所有信息的差异
输出对应关系不同的数据保存文件
如果两个db3全字段相同则不输出文件
"""
from re_common.baselibrary.utils.basedir import BaseDir
from re_common.baselibrary.utils.basefile import BaseFile
from re_common.facade.mysqlfacade import MysqlUtiles
import sqlite3
import operator
import time
class Constrast(object):
    def __init__(self):
        self.curPath = BaseDir.get_file_dir_absolute(__file__)
        self.set_old = set()
        self.set_new = set()


    def read_primary_key(self):
        """
        取出主键，对照其内容，old参照new的rawid
        :return:
        """
        print("对照主键")
        old_path = r'\\192.168.31.177\down_data\cnkiwanfang\zt_wf_qk_20200323\db3\zt_wf_qk_20200323_0001.db3'
        new_path = r'E:\更新数据\db3\zt_wf_qk.20200331_000.db3'
        old = sqlite3.connect(old_path)
        new = sqlite3.connect(new_path)
        sql = "select rawid from modify_title_info_zt"
        old_cur = old.cursor()
        old_cur.execute(sql)
        for item in old_cur.fetchall():
            rawid = item[0]
            self.set_old.add(rawid)
        new_cur = new.cursor()
        new_cur.execute(sql)
        for item in new_cur.fetchall():
            rawid = item[0]
            self.set_new.add(rawid)
        print(len(self.set_new - self.set_old))


    def read_content(self):
        """
        判断主键相同的，内容是否相同，new参照old的内容
        输出content_contrast.txt
        :return:
        """
        def dict_factory(cursor, row):
            d = {}
            for idx, col in enumerate(cursor.description):
                d[col[0]] = row[idx]
            return d

        print("对照内容")
        old_path = r'\\192.168.31.177\down_data\cnkiwanfang\zt_wf_qk_20200323\db3\zt_wf_qk_20200323_0001.db3'
        new_path = r'E:\更新数据\db3\zt_wf_qk.20200331_000.db3'
        old = sqlite3.connect(old_path)
        old.row_factory = dict_factory
        new = sqlite3.connect(new_path)
        new.row_factory = dict_factory

        sql = "select rawid from modify_title_info_zt"
        old_cur = old.cursor()
        new_cur = new.cursor()
        old_cur.execute(sql)
        rawidlist = []

        for item in old_cur.fetchall():

            rawid = item['rawid']
            rawidlist.append(rawid)
            if len(rawidlist) <= 10000:
                continue
            sql = "select * from modify_title_info_zt where rawid in {}".format(tuple(rawidlist))
            # print(sql)
            old_cur.execute(sql)
            new_cur.execute(sql)
            old_message = old_cur.fetchall()
            new_mesaage = new_cur.fetchall()
            dicts_old ={}
            dicts_new = {}
            for itemss in old_message:
                dicts_old[itemss["rawid"]] = itemss
            for items in new_mesaage:
                dicts_new[items["rawid"]] = items

            for k,v in dicts_old.items():
                for kk,vv in v.items():
                    if kk != "batch" and kk != 'gch' and kk != 'subject_clc':
                        if vv != dicts_new[k][kk]:
                            print("rawid:{}".format(k))
                            print("字段名:{}".format(kk))
                            print("old_message:{}".format(vv))
                            print("new_message:{}".format(dicts_new[k][kk]))
                            time.sleep(100)
                    if kk == 'gch' or kk == 'subject_clc':
                        if vv != dicts_new[k][kk] and (vv == "" or dicts_new[k][kk]==""):
                            print("rawid:{}".format(k))
                            print("字段名:{}".format(kk))
                            print("old_message:{}".format(vv))
                            print("new_message:{}".format(dicts_new[k][kk]))
                            time.sleep(100)
            print("*************************************")
            rawidlist.clear()





    def run(self):
        """
        启动程序
        :return:
        """
        # self.read_primary_key()
        self.read_content()

if __name__ == '__main__':
    c = Constrast()
    c.run()




class BaseEncode(object):

    def __init__(self):
        pass

    @classmethod
    def get_md5_value(cls, src):
        """
        获得字符串md5值
        :param src:需要操作的字符串
        :return:
        """
        import hashlib
        myMd5 = hashlib.md5()
        myMd5.update(src.encode("utf8"))
        myMd5_Digest = myMd5.hexdigest()
        return myMd5_Digest

    @classmethod
    def get_sha1_value(cls, src):
        """
         获得字符串sha1值
        :param src:
        :return:
        """
        import hashlib
        mySha1 = hashlib.sha1()
        mySha1.update(src)
        mySha1_Digest = mySha1.hexdigest()
        return mySha1_Digest



# -*- coding: utf-8 -*-
# @Time    : 2019/10/11 14:51
# @Author  : qianjun
# @FileName: ogp.py
# @Software: PyCharm
"""石油地球物理勘探期刊我pdf 下载"""
import re
import sys

import requests
from urllib import parse
import parent
from bs4 import BeautifulSoup

hdrs = {"User-Agent":"Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/75.0.3770.100 Safari/537.36",
        "Referer" :"http://www.ogp-cn.com.cn",

}
Parent = parent.Parent()

class ogp(object):


    def __init__(self):
        self.url = 'http://www.jos.org.cn/jos/ch/reader/issue_browser.aspx'
        self.ero = 0
        self.suc = 0
        self.cnt = 0

        self.journal = '石油地球物理勘探'

    def getpdf(self,years, num,pdfRoot):
        years = str(years)
        num = str(num)
        url = 'http://journal08.magtechjournal.com/Jwk_ogp/CN/article/showTenYearVolumnDetail.do?nian=%s' % (years)

        try:
            r = requests.get(url, headers=hdrs, timeout= 100)
        except:

            print('---------访问页面失败-----------')
            return False
        soup = BeautifulSoup(r.text, 'lxml')
        all_table = soup.select('a[href*="../volumn/volumn_"]')

        href =""
        # 循环取出列表中标签，剔除有img标签的
        for onetable in all_table:
            print("111")
            if onetable.find("img"):
                continue
            else:

                checkStr = "".join(onetable.stripped_strings).replace("\r\n","")

                matchObj = re.match(r'.*?No.(\S+).*?',checkStr)

                if matchObj:
                    print("222")
                    numStr = matchObj.group(1)
                    print(numStr)
                    # 当输入的期次与该值相等时，获取a链接标签,进行pdf下载操作
                    if numStr == num:
                        href = "".join(onetable['href']).replace("..","http://journal08.magtechjournal.com/Jwk_ogp/CN/")

        if not href:
            print("************格式异常*****************")
            return False

        # 重新请求页面
        print("列表url:%s"%href)
        try:
            r = requests.get(href, headers=hdrs, timeout= 100)

        except Exception as e:
            print("--------错误信息：%s-----------"%str(e))
            print('---------访问页面失败-----------')
            return False

        # 获取pdf链接

        soups = BeautifulSoup(r.text, 'lxml')
        pdf_hrefs = soups.select('a[href*="../article/downloadArticleFile.do?attachType=PDF"]')

        pdfnum =0
        self.cnt = len(pdf_hrefs)
        print('journal: %s, vol: %s。共有文章 %d；' % (
            self.journal, num, self.cnt))
        for one in pdf_hrefs:
            pdfnum += 1

            pdfurl ="".join(one['href']).replace("..","http://journal08.magtechjournal.com/Jwk_ogp/CN/")
            title = str(pdfnum)
            print("".join(one['href']))
            print('--------开始下载---------: %s' % title)

            pdfFilePath = Parent.makedir(self.journal, years, str(num), title, pdfRoot)
            if pdfFilePath == 1:

                self.suc += 1
                continue
            # pdf_hdrs = {"Cache-Control":"no-cache",
            #     "Connection":"keep-alive",
            #     "Content-Type":"application/x-download",
            #     "Server":"nginx/1.10.2",
            # "Accept-Encoding":"identity",
            #     "Transfer-Encoding":"chunked",
            #     "X-Frame-Options":"SAMEORIGIN"}
            fig = Parent.getPdf(pdfurl, title, pdfFilePath, hdrs)
            # self.cnt = len(downlist)
            # print("self.cnt:%s"%self.cnt)
            if fig == -1:
                self.ero += 1
                line = 'journal: %s, years: %s, issue: %s: %s 下载失败' % (
                    self.journal, years, num, title.strip())
                line = line + '\n'
                Parent.Record(line, self.journal)
            if fig == 1:
                print('%s,pdf下载成功' % title)
                self.suc += 1

                line = 'journal: %s, years: %s, issue: %s。共有文章 %d；下载PDF %d；失败 %d' % (
                    self.journal, years, num, self.cnt, self.suc, self.ero)
                line = line + '\n'
                Parent.Record(line, self.journal)
        Parent.Record("\n", self.journal)
        return True

# if __name__ == '__main__':
#
#     work = ogp()
#     work.getpdf("2019","4","./")



import sys
import time

import redis
from redis.client import Pipeline, Redis

from re_common.baselibrary import IniConfig


class MyRedis(object):
    def __init__(self, configpath=""):
        self.configpath = configpath

    def set_configpath(self, configpath):
        self.configpath = configpath

    def set_redis_from_config(self, sesc="proxy", encoding="utf-8"):
        """
        设置 redis的配置信息
        :param sesc: 选择配置文件里的字典信息
        :param encoding: 编码
        :return:
        """
        assert self.configpath != "", 'configpath 为空，请调用set_configpath'
        dictsall = IniConfig(self.configpath).get_conf_dict(encoding=encoding)
        dicts = dictsall[sesc]
        self.RedisHost = dicts['RedisHost']
        self.RedisPort = dicts['RedisPort']
        self.RedisDB = dicts['RedisDB']
        self.RedisKey = dicts['RedisKey']

    def conn_redis(self):
        """
        redis 提供两个类 Redis 和 StrictRedis, StrictRedis 用于实现大部分官方的命令，
        Redis 是 StrictRedis 的子类，用于向后兼用旧版本。
        redis 取出的结果默认是字节，我们可以设定 decode_responses=True 改成字符串。
        链接 设置好的配置文件的redis
        :return: 返回一个 connect
        """
        assert self.RedisHost, 'RedisHost 不存在，请先调用set_redis_from_config'
        assert self.RedisPort, 'RedisPort 不存在，请先调用set_redis_from_config'
        assert self.RedisDB, 'RedisDB 不存在，请先调用set_redis_from_config'
        assert self.RedisKey, 'RedisKey 不存在，请先调用set_redis_from_config'
        self.rconn = redis.StrictRedis(host=self.RedisHost, port=self.RedisPort, db=self.RedisDB, decode_responses=True)
        return self.rconn

    def get_pipeline(self):
        """
        获取一个通道，便于批量删除和增加（节约时耗）
        redis默认在执行每次请求都会创建（连接池申请连接）和断开（归还连接池）一次连接操作，
        如果想要在一次请求中指定多个命令，则可以使用pipline实现一次请求指定多个命令，
        并且默认情况下一次pipline 是原子性操作。
        管道（pipeline）是redis在提供单个请求中缓冲多条服务器命令的基类的子类。
        它通过减少服务器-客户端之间反复的TCP数据库包，从而大大提高了执行批量命令的功能。
        :return: pipeline()
        """
        assert isinstance(self.rconn, Redis), Exception("请调用conn_redis获取")
        self.pipe = self.rconn.pipeline()
        return self.pipe

    def set_conn_or_pipe(self, bools=True):
        assert self.rconn, 'rconn 不存在，请先调用 conn_redis'
        assert self.pipe, 'pipe 不存在，请先调用 get_pipeline'
        if bools:
            self.r = self.rconn
        else:
            self.r = self.pipe

    def get_conn_pool(self):
        """
        获取redis连接池
        :return:
        """
        self.pool = redis.ConnectionPool(host=self.RedisHost, port=self.RedisPort,
                                         db=self.RedisDB, decode_responses=True)
        return self.pool

    def conn_redis_from_pool(self, pool):
        self.rconn = redis.StrictRedis(connection_pool=pool)
        return self.rconn

    def builder(self, configpath=None, sesc="proxy", encoding="utf-8", conn_or_pipe=True):
        """
        一次性设置好所有变量，builder模式
        :param configpath:
        :param sesc:
        :param encoding:
        :param conn_or_pipe:
        :return:
        """
        if configpath:
            self.set_configpath(configpath=configpath)
        self.set_redis_from_config(sesc=sesc, encoding=encoding)
        self.conn_redis()
        self.get_pipeline()
        self.set_conn_or_pipe(conn_or_pipe)

    def getDataFromRedis(self):
        """
        获取 信息 获取全部数据
        :return: 一个可迭代的数据
        """
        assert self.RedisKey, 'RedisKey 不存在，请先调用set_redis_from_config'
        assert self.r, 'r 不存在，请先调用 set_conn_or_pipe'
        if self.r:
            rows = self.r.smembers(self.RedisKey)
            return rows
        else:
            print("redis出现连接错误")
            sys.exit(-1)

    def delete(self, RedisKey):
        """
        删除一个RedisKey
        :param RedisKey: 需要删除的RedisKey
        :return:
        """
        assert self.r, 'r 不存在，请先调用 set_conn_or_pipe'
        self.r.delete(RedisKey)

    def set(self, name, value, ex=None, px=None, nx=False, xx=False, keepttl=False):
        """
        写入 在 Redis 中设置值，默认，不存在则创建，存在则修改。
        :param name:
        :param value:
        :param ex: 过期时间（秒）
        :param px: 过期时间（毫秒）
        :param nx: 如果设置为True，则只有name不存在时，当前set操作才执行
        :param xx: 如果设置为True，则只有name存在时，当前set操作才执行
        :param keepttl:
        :return:
        """
        assert self.r, 'r 不存在，请先调用 set_conn_or_pipe'
        return self.r.set(name, value, ex=ex, px=px, nx=nx, xx=xx, keepttl=keepttl)

    def get(self, name):
        """
        取出
        :param name:
        :return:
        """
        assert self.r, 'r 不存在，请先调用 set_conn_or_pipe'
        return self.r.get(name)

    def setnx(self, name, value):
        """
        设置值，只有name不存在时，执行设置操作（添加）
        :param name:
        :param value:
        :return:
        """
        assert self.r, 'r 不存在，请先调用 set_conn_or_pipe'
        return self.r.setnx(name, value)

    def setex(self, name, time, value):
        """
        # time秒后，取值就从value变成None
        :param name:
        :param time: 过期时间（数字秒 或 timedelta对象）
        :param value:
        :return:
        """
        assert self.r, 'r 不存在，请先调用 set_conn_or_pipe'
        return self.r.setex(name, time, value)

    def psetex(self, name, time_ms, value):
        """
        # time_ms毫秒后，取值就从value变成None
        :param name:
        :param time_ms:  过期时间（数字毫秒 或 timedelta对象）
        :param value:
        :return:
        """
        assert self.r, 'r 不存在，请先调用 set_conn_or_pipe'
        return self.r.psetex(name, time_ms, value)

    def mset(self, mapping):
        """
        批量设置值
        :param args:
        :param kwargs:
        :return:
        """
        assert self.r, 'r 不存在，请先调用 set_conn_or_pipe'
        return self.r.mset(mapping=mapping)

    def mget(self, keys, *args):
        """
        批量获取

        :param keys: ['k1', 'k2'] or "fruit", "fruit1", "fruit2", "k1", "k2"
        :param args:
        :return:
        """
        assert self.r, 'r 不存在，请先调用 set_conn_or_pipe'
        return self.r.mget(keys, *args)

    def getset(self, name, value):
        """
        设置新值并获取原来的值
        设置的新值是value 设置前的值是原来的value
        :param name:
        :param value:
        :return:
        """
        assert self.r, 'r 不存在，请先调用 set_conn_or_pipe'
        return self.r.getset(name, value)

    def getrange(self, key, start, end):
        """
        获取子序列（根据字节获取，非字符）
        r.set("cn_name", "君惜大大") # 汉字
        getrange("cn_name", 0, 2) # 取索引号是0-2 前3位的字节 君 切片操作 （一个汉字3个字节 1个字母一个字节 每个字节8bit）
        getrange("cn_name", 0, -1) # 取所有的字节 君惜大大 切片操作
        r.set("en_name","junxi") # 字母
        getrange("en_name", 0, 2) # 取索引号是0-2 前3位的字节 jun 切片操作 （一个汉字3个字节 1个字母一个字节 每个字节8bit）
        getrange("en_name", 0, -1) # 取所有的字节 junxi 切片操作
        :param key:  Redis 的 name
        :param start: 起始位置（字节）
        :param end:  结束位置（字节）
        :return:
        """
        assert self.r, 'r 不存在，请先调用 set_conn_or_pipe'
        return self.r.getrange(key, start, end)

    def setrange(self, name, offset, value):
        """
        修改字符串内容，从指定字符串索引开始向后替换（新值太长时，则向后添加）
        r.setrange("en_name", 1, "ccc")
        # jccci 原始值是junxi 从索引号是1开始替换成ccc 变成 jccci
        :param name:
        :param offset: 字符串的索引，字节（一个汉字三个字节）
        :param value: 要设置的值
        :return:
        """
        assert self.r, 'r 不存在，请先调用 set_conn_or_pipe'
        return self.r.setrange(name, offset, value)

    def setbit(self, name, offset, value):
        """
        注：如果在Redis中有一个对应： n1 = "foo"，

        那么字符串foo的二进制表示为：01100110 01101111 01101111

        所以，如果执行 setbit('n1', 7, 1)，则就会将第7位设置为1，

        那么最终二进制则变成 01100111 01101111 01101111，即："goo"

        source = "foo"
        for i in source:
            num = ord(i)

        特别的，如果source是汉字 "陈思维"怎么办？
        对于utf-8，每一个汉字占 3 个字节，那么 "陈思维" 则有 9个字节 对于汉字，for循环时候会按照 字节 迭代，那么在迭代时，将每一个字节转换 十进制数，然后再将十进制数转换成二进制
        :param name: redis的name
        :param offset: 位的索引（将值变换成二进制后再进行索引）
        :param value: 值只能是 1 或 0
        :return:
        """
        assert self.r, 'r 不存在，请先调用 set_conn_or_pipe'
        return self.r.setbit(name, offset, value)

    def getbit(self, name, offset):
        """
        获取bit
        :param name:
        :param offset:
        :return:
        """
        assert self.r, 'r 不存在，请先调用 set_conn_or_pipe'
        return self.r.getbit(name, offset)

    def bitcount(self, key, start=None, end=None):
        """
        获取name对应的值的二进制表示中 1 的个数
        :param key: Redis的name
        :param start: 字节起始位置
        :param end: 字节结束位置
        :return:
        """
        assert self.r, 'r 不存在，请先调用 set_conn_or_pipe'
        return self.r.bitcount(key, start, end)

    def bitop(self, operation, dest, *keys):
        """
        获取多个值，并将值做位运算，将最后的结果保存至新的name对应的值
        获取Redis中n1,n2,n3对应的值，然后讲所有的值做位运算（求并集），然后将结果保存 new_name 对应的值中
        :param operation: AND（并） 、 OR（或） 、 NOT（非） 、 XOR（异或）
        :param dest: 新的Redis的name
        :param keys: 要查找的Redis的name
        :return:
        """
        assert self.r, 'r 不存在，请先调用 set_conn_or_pipe'
        return self.r.bitop(operation, dest, *keys)

    def strlen(self, name):
        """
        返回name对应值的字节长度（一个汉字3个字节）
        :param name:
        :return:
        """
        assert self.r, 'r 不存在，请先调用 set_conn_or_pipe'
        return self.r.strlen(name)

    def incr(self, name, amount=1):
        """
        自增 name 对应的值，当 name 不存在时，则创建 name＝amount，否则，则自增。
        :param name:  Redis的name
        :param amount:  自增数（必须是整数）
        :return:
        """
        assert self.r, 'r 不存在，请先调用 set_conn_or_pipe'
        return self.r.incr(name, amount)

    def incrbyfloat(self, name, amount=1.0):
        """
        自增 name对应的值，当name不存在时，则创建name＝amount，否则，则自增。
        :param name: Redis的name
        :param amount: 自增数（浮点型）
        :return:
        """
        assert self.r, 'r 不存在，请先调用 set_conn_or_pipe'
        return self.r.incrbyfloat(name, amount)

    def decr(self, name, amount=1):
        """
        自减 name 对应的值，当 name 不存在时，则创建 name＝amount，否则，则自减。
        :param name: Redis的name
        :param amount: 自减数（整数)
        :return:
        """
        assert self.r, 'r 不存在，请先调用 set_conn_or_pipe'
        return self.r.decr(name, amount)

    def append(self, key, value):
        """
        在redis name对应的值后面追加内容
        :param key: redis的name
        :param value: 要追加的字符串
        :return:
        """
        assert self.r, 'r 不存在，请先调用 set_conn_or_pipe'
        return self.r.append(key, value)

    ################redis 基本命令 hash#########

    def hset(self, name, key, value):
        """
        name对应的hash中设置一个键值对（不存在，则创建；否则，修改）

        :param name: redis的name
        :param key: name对应的hash中的key
        :param value: name对应的hash中的value
        :return:
        """
        assert self.r, 'r 不存在，请先调用 set_conn_or_pipe'
        return self.r.hset(name, key, value)

    def hsetnx(self, name, key, value):
        """
        hsetnx(name, key, value) 当name对应的hash中不存在当前key时则创建（相当于添加）
        hsetnx   # 只能新建
        :param name:
        :param key:
        :param value:
        :return:
        """
        assert self.r, 'r 不存在，请先调用 set_conn_or_pipe'
        return self.r.hsetnx(name, key, value)

    def hget(self, name, key):
        """
        单个取hash的key对应的值
        :param name:
        :param key:
        :return:
        """
        assert self.r, 'r 不存在，请先调用 set_conn_or_pipe'
        return self.r.hget(name, key)

    def hmset(self, name, mapping):
        """
        在name对应的hash中批量设置键值对
        :param name:  redis的name
        :param mapping:  字典，如：{'k1':'v1', 'k2': 'v2'}
        :return:
        """
        assert self.r, 'r 不存在，请先调用 set_conn_or_pipe'
        return self.r.hmset(name, mapping)

    def hmget(self, name, keys, *args):
        """
        在name对应的hash中获取多个key的值
        :param name: reids对应的name
        :param keys: 要获取key集合，如：['k1', 'k2', 'k3']
        :param args: 要获取的key，如：k1,k2,k3
        :return:
        """
        assert self.r, 'r 不存在，请先调用 set_conn_or_pipe'
        return self.r.hmget(name, keys, *args)

    def hgetall(self, name):
        """
        获取name对应hash的所有键值
        :param name:
        :return:
        """
        assert self.r, 'r 不存在，请先调用 set_conn_or_pipe'
        return self.r.hgetall(name)

    def hlen(self, name):
        """
        获取name对应的hash中键值对的个数
        :param name:
        :return:
        """
        assert self.r, 'r 不存在，请先调用 set_conn_or_pipe'
        return self.r.hlen(name)

    def hkeys(self, name):
        """
        得到所有的keys（类似字典的取所有keys）
        :param name:
        :return:
        """
        assert self.r, 'r 不存在，请先调用 set_conn_or_pipe'
        return self.r.hkeys(name)

    def hvals(self, name):
        """
        得到所有的value（类似字典的取所有value）
        :param name:
        :return:
        """
        assert self.r, 'r 不存在，请先调用 set_conn_or_pipe'
        return self.r.hvals(name)

    def hexists(self, name, key):
        """
        判断成员是否存在（类似字典的in）
        检查 name 对应的 hash 是否存在当前传入的 key
        :param name:
        :param key:
        :return:
        """
        assert self.r, 'r 不存在，请先调用 set_conn_or_pipe'
        return self.r.hexists(name)

    def hdel(self, name, *keys):
        """
        删除键值对
        :return:
        """
        assert self.r, 'r 不存在，请先调用 set_conn_or_pipe'
        return self.r.hdel(name, *keys)

    def hincrby(self, name, key, amount=1):
        """
        自增name对应的hash中的指定key的值，
        不存在则创建key=amount
        自增自减整数(将key对应的value--整数
        自增1或者2，或者别的整数 负数就是自减)
        :param name: redis中的name
        :param key:  hash对应的key
        :param amount: 自增数（整数）
        :return:
        """
        assert self.r, 'r 不存在，请先调用 set_conn_or_pipe'
        return self.r.hincrby(name, key, amount)

    def hincrbyfloat(self, name, key, amount=1.0):
        """
        自增name对应的hash中的指定key的值，不存在则创建key=amount
        自增自减浮点数(将key对应的value--浮点数 自增1.0或者2.0，
        或者别的浮点数 负数就是自减)
        :param name: redis中的name
        :param key: hash对应的key
        :param amount: amount，自增数（浮点数）
        :return:
        """
        assert self.r, 'r 不存在，请先调用 set_conn_or_pipe'
        return self.r.hincrbyfloat(name, key, amount)

    def hscan(self, name, cursor=0, match=None, count=None):
        """
        增量式迭代获取，对于数据大的数据非常有用，hscan可以实现分片的获取数据，并非一次性将数据全部获取完，从而放置内存被撑爆
        如:
        第一次：cursor1, data1 = r.hscan('xx', cursor=0, match=None, count=None)
        第二次：cursor2, data1 = r.hscan('xx', cursor=cursor1, match=None, count=None)
        直到返回值cursor的值为0时，表示数据已经通过分片获取完毕
        :param name: redis的name
        :param cursor: 游标（基于游标分批取获取数据）
        :param match: 匹配指定key，默认None 表示所有的key
        :param count: 每次分片最少获取个数，默认None表示采用Redis的默认分片个数
        :return:
        """
        assert self.r, 'r 不存在，请先调用 set_conn_or_pipe'
        return self.r.hscan(name, cursor, match, count)

    def hscan_iter(self, name, match=None, count=None):
        """
        利用yield封装hscan创建生成器，实现分批去redis中获取数据

        :param name:
        :param match: 匹配指定key，默认None 表示所有的key
        :param count: 每次分片最少获取个数，默认None表示采用Redis的默认分片个数
        :return: # 生成器内存地址 for循环进行迭代
        """
        assert self.r, 'r 不存在，请先调用 set_conn_or_pipe'
        return self.r.hscan_iter(name, match, count)

    ####################redis基本命令 list #######

    def push(self, name, *args):
        """
        增加（类似于list的append，只是这里是从左边新增加）--没有就新建
        r.lpush("list1", 11, 22, 33)
        保存顺序为: 33,22,11
        :param name:
        :param values:
        :return:
        """
        assert self.r, 'r 不存在，请先调用 set_conn_or_pipe'
        return self.r.lpush(name, *args)

    def rpush(self, name, *args):
        """
        增加（从右边增加）--没有就新建
        r.rpush("list2", 44, 55, 66)
        # 在列表的右边，依次添加44,55,66
        :param name:
        :param args:
        :return:
        """
        assert self.r, 'r 不存在，请先调用 set_conn_or_pipe'
        return self.r.rpush(name, *args)

    def lpushx(self, name, value):
        """
        往已经有的name的列表的左边添加元素，没有的话无法创建
        :param name:
        :param value:
        :return:
        """
        assert self.r, 'r 不存在，请先调用 set_conn_or_pipe'
        return self.r.lpushx(name, value)

    def rpushx(self, name, value):
        """
        往已经有的name的列表的右边添加元素，没有的话无法创建
        :param name:
        :param value:
        :return:
        """
        assert self.r, 'r 不存在，请先调用 set_conn_or_pipe'
        return self.r.rpushx(name, value)

    def linsert(self, name, where, refvalue, value):
        """
        r.linsert("list2", "before", "11", "00")
        # 往列表中左边第一个出现的元素"11"前插入元素"00"
        :param name:
        :param where: BEFORE或AFTER
        :param refvalue: 标杆值，即：在它前后插入数据
        :param value: 要插入的数据
        :return:
        """
        assert self.r, 'r 不存在，请先调用 set_conn_or_pipe'
        return self.r.linsert(name, where, refvalue, value)

    def lset(self, name, index, value):
        """
        对name对应的list中的某一个索引位置重新赋值
        r.lset("list2", 0, -11)    # 把索引号是0的元素修改成-11
        :param name:
        :param index: list的索引位置
        :param value: 要设置的值
        :return:
        """
        assert self.r, 'r 不存在，请先调用 set_conn_or_pipe'
        return self.r.lset(name, index, value)

    def lrem(self, name, value, num):
        """
        在name对应的list中删除指定的值
        :param name:
        :param value: 要删除的值
        :param num:  num=0，删除列表中所有的指定值
        num=2 - 从前到后，删除2个, num=1,从前到后，删除左边第1个
        num=-2 - 从后向前，删除2个
        :return:
        """
        assert self.r, 'r 不存在，请先调用 set_conn_or_pipe'
        return self.r.lrem(name, value, num)

    def lpop(self, name):
        """
        在name对应的列表的左侧获取第一个元素并在列表中移除，返回值则是第一个元素
        :param name:
        :return:
        """
        assert self.r, 'r 不存在，请先调用 set_conn_or_pipe'
        return self.r.lpop(name)

    def rpop(self, name):
        """
        在name对应的列表的右侧获取第一个元素并在列表中移除，返回值则是第一个元素
        rpop(name) 表示从右向左操作
        :param name:
        :return:
        """
        assert self.r, 'r 不存在，请先调用 set_conn_or_pipe'
        return self.r.rpop(name)

    def ltrim(self, name, start, end):
        """
        删除索引之外的值
        r.ltrim("list2", 0, 2) # 删除索引号是0-2之外的元素，
        值保留索引号是0-2的元素
        :param name:
        :param start: 索引的起始位置
        :param end: 索引的起始位置
        :return:
        """
        assert self.r, 'r 不存在，请先调用 set_conn_or_pipe'
        return self.r.ltrim(name, start, end)

    def lindex(self, name, index):
        """
        取值（根据索引号取值）
        r.lindex("list2", 0)  # 取出索引号是0的值
        :param name:
        :param index:
        :return:
        """
        assert self.r, 'r 不存在，请先调用 set_conn_or_pipe'
        return self.r.lindex(name, index)

    def rpoplpush(self, src, dst):
        """
        移动 元素从一个列表移动到另外一个列表
        从一个列表取出最右边的元素，同时将其添加至另一个列表的最左边
        :param src:  要取数据的列表的 name
        :param dst:   要添加数据的列表的 name
        :return:
        """
        assert self.r, 'r 不存在，请先调用 set_conn_or_pipe'
        return self.r.rpoplpush(src, dst)

    def brpoplpush(self, src, dst, timeout=0):
        """
        移动 元素从一个列表移动到另外一个列表 可以设置超时
        :param src: 取出并要移除元素的列表对应的name
        :param dst: 要插入元素的列表对应的name
        :param timeout: 当src对应的列表中没有数据时，阻塞等待其有数据的超时时间（秒），0 表示永远阻塞
        :return:
        """
        assert self.r, 'r 不存在，请先调用 set_conn_or_pipe'
        return self.r.brpoplpush(src, dst, timeout)

    def blpop(self, keys, timeout=0):
        """
        将多个列表排列，按照从左到右去pop对应列表的元素
        r.blpop(["list10", "list11"], timeout=2)
        :param keys: redis的name的集合
        :param timeout: 超时时间，当元素所有列表的元素获取完之后，阻塞等待列表内有数据的时间（秒）, 0 表示永远阻塞
        :return:
        """
        assert self.r, 'r 不存在，请先调用 set_conn_or_pipe'
        return self.r.blpop(keys, timeout)

    def list_iter(self, name):
        """
        由于redis类库中没有提供对列表元素的增量迭代，如果想要循环name对应的列表的所有元素，那么就需要获取name对应的所有列表。

        循环列表

        但是，如果列表非常大，那么就有可能在第一步时就将程序的内容撑爆，所有有必要自定义一个增量迭代的功能：
        自定义redis列表增量迭代
        # 使用
        for item in list_iter('list2'): # 遍历这个列表
            print(item)
        :param name: redis中的name，即：迭代name对应的列表
        :return: yield 返回 列表元素
        """
        assert self.r, 'r 不存在，请先调用 set_conn_or_pipe'
        list_count = self.r.llen(name)
        for index in range(list_count):
            yield self.r.lindex(name, index)

    ################redis基本命令 set#############

    def sadd(self, name, *values):
        """
        增加 一个 set集合
        :param name: 需要增加的rediskey
        :param ProxyPoolValid: 传入集合
        :return:
        """
        assert self.r, 'r 不存在，请先调用 set_conn_or_pipe'
        return self.r.sadd(name, *values)

    def scard(self, name):
        """
        获取name对应的集合中元素个数
        :param name:
        :return:
        """
        assert self.r, 'r 不存在，请先调用 set_conn_or_pipe'
        return self.r.scard(name)

    def smembers(self, name):
        """
        获取集合中所有的成员
        :param name:
        :return:
        """
        assert self.r, 'r 不存在，请先调用 set_conn_or_pipe'
        return self.r.smembers(name)

    def sscan(self, name, cursor=0, match=None, count=None):
        """
        获取集合中所有的成员--元组形式
        :param name:
        :return:
        """
        assert self.r, 'r 不存在，请先调用 set_conn_or_pipe'
        return self.r.sscan(name, cursor, match, count)

    def sscan_iter(self, name, match=None, count=None):
        """
        获取集合中所有的成员--迭代器的方式
        同字符串的操作，用于增量迭代分批获取元素，避免内存消耗太大
        :param name:
        :param match:
        :param count:
        :return:
        """
        assert self.r, 'r 不存在，请先调用 set_conn_or_pipe'
        return self.r.sscan_iter(name, match, count)

    def sdiff(self, keys, *args):
        """
        差集
        r.sdiff("set1", "set2") # 在集合set1但是不在集合set2中
        r.sdiff("set2", "set1") # 在集合set2但是不在集合set1中
        :param keys:
        :param args:
        :return:
        """
        assert self.r, 'r 不存在，请先调用 set_conn_or_pipe'
        return self.r.sdiff(keys, *args)

    def sdiffstore(self, dest, keys, *args):
        """
        差集--差集存在一个新的集合中
        获取第一个name对应的集合中且不在其他name对应的集合，再将其新加入到dest对应的集合中
         r.sdiffstore("set3", "set1", "set2")  # 在集合set1但是不在集合set2中
        :param dest:
        :param keys:
        :param args:
        :return:
        """
        assert self.r, 'r 不存在，请先调用 set_conn_or_pipe'
        return self.r.sdiffstore(dest, keys, *args)



def RedisConnect(configpath, sesc="proxy", encoding="utf-8"):
    """
    连接数据库 通过读取配置文件连接,如果读取配置文件 失败  返回None
    :return:
    """
    dictsall = IniConfig(configpath).get_conf_dict(encoding=encoding)
    dicts = dictsall[sesc]
    RedisHost = dicts['RedisHost']
    RedisPort = dicts['RedisPort']
    RedisDB = dicts['RedisDB']
    RedisKey = dicts['RedisKey']
    try:
        rconn = redis.StrictRedis(host=RedisHost, port=RedisPort, db=RedisDB, decode_responses=True)
    except:
        # 有可能因为网络波动无法连接 这里休眠10秒重连一次  如果还是失败就放弃
        time.sleep(10)
        rconn = redis.StrictRedis(host=RedisHost, port=RedisPort, db=RedisDB, decode_responses=True)
    if rconn:
        return rconn, RedisKey
    return None


def getDataFromRedis(configpath, sesc="proxy"):
    """
    取出数据
    :param configpath:
    :param sesc:
    :return:
    """
    rconn, RedisKey = RedisConnect(configpath, sesc=sesc)
    if rconn:
        rows = rconn.smembers(RedisKey)
        return rows
    else:
        print("redis出现连接错误")
        sys.exit(-1)



from faker import Faker
from faker.providers import internet

fake = Faker()
# 生成名字
print(fake.name())
# 生成地址
print(fake.address())
# 生成国家
print(fake.country())
# 生成省份
print(fake.province())
# 生成市　县
print(fake.city_suffix())
# 生成区
print(fake.district())
# 生成街道名
print(fake.street_name())
# 生成街　路
print(fake.street_suffix())
#　生成０～９　随机数
print(fake.random_digit())
# 生成随机字母
print(fake.random_element())
# 生成随机颜色名
print(fake.color_name())
# 生成随机日期
print(fake.date())
# 生成一段文字
print(fake.text())

fake.add_provider(internet)

# 生成ipv4
print(fake.ipv4_private())




import threading
import time
from abc import ABC, abstractmethod

from re_common.baselibrary.mthread.mythreading import ThreadPoolManger
from re_common.baselibrary.utils.ringlist import RingList

from re_common.baselibrary.mthread.MThreadingRun import especialThreadName
from re_common.facade.loggerfacade import get_streamlogger

"""
event版的多线程，现在已经不在使用 (已废弃)
"""
class MThreadingRunEvent(ABC):
    def __init__(self, num, logger=None):
        if logger:
            self.logger = logger
        else:
            self.logger = get_streamlogger()
        self.etn = especialThreadName()
        # 线程数
        self.threadingnum = num
        # 代理列表
        self.list_proxy = RingList()
        # 线程池
        self.thread_pool = ThreadPoolManger(self.threadingnum, self.logger)
        self.thread_pool.set_callback(self.thread_pool_hook)
        # 结果集
        self.results = []
        # 线程方法
        self.func = self.fun
        # 结果数
        self.resultnum = 0
        # 工作线程数
        self.jobnum = 0
        # 结果集被处理标志  默认被处理 是为了兼容之前的代码不去改动
        self.dealresultstatus = True
        # 結果到达该数量后处理结果 默认及时处理
        self.dealresultsnum = 0
        # 代理设置时间
        self.proxytime = 0
        # 标识event set之后是否对任务进行了设置
        # self.eventbool = False
        self.modle = 1
        # 在任务和处理结果时event信号的状态
        self.result_event_status = True
        self.task_event_status = True
        self.thread_run_lock = threading.Lock()

        # 全局使用特殊的单词
        self.BREAK = "break"

    def set_is_restart(self, name, is_restart):
        self.thread_pool.especial_thread_pool_dicts[name].set_is_restart(is_restart)

    @abstractmethod
    def setProxy(self, proxysList=None):
        """
        将代理加入到循环队列中 self.list_proxy
        :param proxysList:
        :return:
        """
        pass

    @abstractmethod
    def fun(self, threadval, *args, **kwargs):
        pass

    @abstractmethod
    def thread_pool_hook(self, thread_pool_dicts, thread, args, kwargs) -> dict:
        """
        钩子函数 可以被重写
        主要重写里面的dicts部分
        :return:
        """
        return {}

    @abstractmethod
    def setTask(self, results=None, *args, **kwargs):
        # self.thread_pool.event.set()  # 自动释放信号
        # 当设置任务和处理结果使用同一个连接时尽量使用event信号保证不同时执行sql在一个执行未返回时
        # 请查询较大数据时主动释放
        pass

    @abstractmethod
    def getTask(self, *args, **kwargs):
        """
        主要用于mysql的请求
        :param args:
        :param kwargs:
        :return:
        """
        pass

    def setTask_noevent(self, *args, **kwargs):
        """
        设置任务，在没有event限制的情况下将两个步骤写在一起
        :param args:
        :param kwargs:
        :return:
        """
        while True:
            results = self.getTask(*args, **kwargs)
            reslt = self.setTask(results, *args, **kwargs)
            if reslt == self.BREAK:
                break

    def __setTask(self, *args, **kwargs):
        # 设置等待任务队列为空时继续添加任务
        # 继承类必须在第一步调用父类的该方法
        while True:
            # 获取Event 状态
            self.task_event_status = True
            self.logger.info("self.thread_pool.event_is_set() is {}".format(self.thread_pool.event_is_set()))
            # 如果状态为False 即　堵塞状态
            if not self.thread_pool.event_is_set():
                # 判断　result_event_status　和　task_event_status　都堵塞　理论上不存在　但Event并不完美
                if self.result_event_status and self.task_event_status:
                    # 设置　信号允许堵塞通过　状态为True
                    self.thread_pool.event_set()
                time.sleep(1)
                continue
            self.logger.info("event_wait")
            # 等待信号后执行设置任务
            self.thread_pool.event_wait(50)
            # 得到信号后clrar 不允许其他地方运行 状态　False
            self.thread_pool.event_clear()
            if self.result_event_status is False:
                self.logger.info("elf.result_event_status is False")
                continue
            self.task_event_status = False
            # 执行getTask
            self.logger.info("getTask")
            result = self.getTask(*args, **kwargs)
            # 运行完毕 其他地方等待的信号可以继续运行 True
            self.thread_pool.event_set()
            self.task_event_status = True
            # 执行ste()
            reslt = self.setTask(result, *args, **kwargs)

            # 是否跳出循环　结束线程
            if reslt == "break":
                break
            time.sleep(1)

    def dealresult(self, *args, **kwargs):
        """
        如果没有在这里处理 请将 self.dealresultstatus = False这样不会丢数据
        :param args:
        :param kwargs:
        :return:
        """
        pass

    def deal_results_no_event(self, *args, **kwargs):
        """
        该函数用于没有信号的机制里
        使用信号主要是因为多线程无法同时使用一个链接
        有可能出现错误
        :param args:
        :param kwargs:
        :return:
        """
        while True:
            # 处理结果标识
            self.dealresultstatus = True
            # 从结果队列获取结果到results
            self.getreustlFromQueue()
            if len(
                    self.results) > self.dealresultsnum or self.thread_pool.work_queue.is_empty() or not self.thread_pool.thread_queue:
                if len(self.results) > 0 or not self.thread_pool.thread_queue.is_empty():
                    # 处理结果
                    self.dealresult()
                    if self.dealresultstatus:
                        # 处理完结果后要清理
                        self.results.clear()
                else:
                    time.sleep(10)
            else:
                time.sleep(3)

    def deal_results(self, *args, **kwargs):
        """
        现在单开线程处理结果
        :return:
        """
        self.logger.info("开始处理结果")
        while True:
            self.result_event_status = True
            self.logger.info("self.thread_pool.event_is_set() is {}".format(self.thread_pool.event_is_set()))
            # 处理结果标识
            self.dealresultstatus = True
            # 从结果队列获取结果到results
            self.getreustlFromQueue()
            if len(self.results) > self.dealresultsnum or self.thread_pool.work_queue.is_empty() \
                    or not self.thread_pool.thread_queue.is_empty():
                if len(self.results) > 0 or not self.thread_pool.thread_queue.is_empty():
                    # 处理结果
                    # 为防止同一个连接多线程操作出现问题使用信号加锁
                    # 等待event为true
                    if not self.thread_pool.event_is_set():
                        # print("deal result is event: " + str(self.thread_pool.event.is_set()))
                        if self.result_event_status and self.task_event_status:
                            self.thread_pool.event_set()
                        time.sleep(1)
                        continue
                    self.thread_pool.event_wait(60)
                    # 设置event为false不允许其他等待线程操作
                    self.thread_pool.event_clear()
                    if self.task_event_status is False:
                        continue
                    self.result_event_status = False
                    # 处理结果
                    self.dealresult()
                    # 本地操作完毕 允许其他线程操作mysql
                    self.thread_pool.event_set()
                    self.result_event_status = True
                    if self.dealresultstatus:
                        # 处理完结果后要清理
                        self.results.clear()
                else:
                    time.sleep(10)
            else:
                time.sleep(3)

    def setfunc(self, func):
        # 设置线程方法
        self.func = func

    def add_job(self, func, *args, **kwargs):
        self.jobnum += 1
        self.thread_pool.add_job(func, *args, **kwargs)

    def getreustlFromQueue(self):
        # 从结果队列获取结果到results
        once_result_num = 0
        while not self.thread_pool.result_queue.is_empty():
            self.resultnum += 1
            once_result_num += 1
            if once_result_num > 100:
                return
            result = self.thread_pool.result_queue.get()
            self.results.append(result)
            self.thread_pool.result_queue.task_done()

    def checkResultsfininsh(self, *args, **kwargs):
        """
        该函数用于处理运行到最后时结果不足100的情况
        :return:
        """
        if self.thread_pool.work_queue.is_empty():
            t1 = len(self.results)
            self.logger.info("self.results len is %s " % str(t1))
            if t1 != 0:
                return False
            else:
                return True

    def other(self):
        self.logger.info("工作队列任务量为{},结果队列任务量为{}".format(self.thread_pool.work_queue.get_size(),
                                                        self.thread_pool.result_queue.get_size()))
        # 通过工作队列和结果队列观察是否结束
        if self.thread_pool.checkThreadRunFinish():
            self.logger.info("初次判断任务已经结束，各个队列为空")
            return True
        else:
            return False

    def check_especial_thread(self):
        if self.modle == 1:
            task = self.__setTask
            proxy = self.setProxy
            result = self.deal_results
        elif self.modle == 2:
            task = self.setTask_noevent
            proxy = self.setProxy
            result = self.deal_results_no_event
        else:
            raise Exception("不存在指定model")

        nowThreadsName = self.thread_pool.get_now_thread()
        for name in list(self.thread_pool.especial_thread_pool_dicts.keys()):
            thread = self.thread_pool.especial_thread_pool_dicts[name].get_thread()
            # 如果线程字典为空 代表已被删除
            if name in nowThreadsName and thread.is_alive():
                # print(name + ": is run")
                pass  # 当前某线程名包含在初始化线程组中，可以认为线程仍在运行
            else:
                self.logger.info("name is :" + name + "; 没有在线程中")
                if name in self.etn.list_name():
                    if name == self.etn.taskthreadname:
                        taskin = task
                    elif name == self.etn.proxythreadname:
                        taskin = proxy
                    elif name == self.etn.dealresultthreadname:
                        taskin = result
                    else:
                        raise Exception("没有对应的任务，请检查")
                    is_start = False
                    if name in self.thread_pool.especial_thread_pool_dicts:
                        threadinfo = self.thread_pool.especial_thread_pool_dicts[name]
                        if threadinfo.get_thread().is_alive():
                            is_start = True
                        if not threadinfo.get_is_restart():
                            is_start = True
                    if not is_start:
                        args = self.thread_pool.especial_thread_pool_dicts[name].get_args()
                        kwargs = self.thread_pool.especial_thread_pool_dicts[name].get_kwargs()
                        self.thread_pool.set_add_especial_thread(taskin, name, *args, **kwargs)

    def start_especial_thread(self):
        # 开启一个线程设置任务
        self.thread_pool.set_add_especial_thread(self.__setTask, self.etn.taskthreadname)
        self.thread_pool.set_add_especial_thread(self.setProxy, self.etn.proxythreadname)
        self.thread_pool.set_add_especial_thread(self.deal_results, self.etn.dealresultthreadname)
        self.thread_pool.especial_start()

    def start_especial_thread_no_evnt(self):
        # 开启一个线程设置任务
        self.thread_pool.set_add_especial_thread(self.setTask_noevent, self.etn.taskthreadname)
        self.thread_pool.set_add_especial_thread(self.setProxy, self.etn.proxythreadname)
        self.thread_pool.set_add_especial_thread(self.deal_results_no_event, self.etn.dealresultthreadname)
        self.thread_pool.especial_start()

    def is_break(self):
        return False

    def run(self, model=1):
        self.modle = model
        if model == 1:
            # 有event
            self.start_especial_thread()
        elif model == 2:
            self.start_especial_thread_no_evnt()
        while True:
            time.sleep(3)
            self.thread_pool.checkThread()
            self.check_especial_thread()
            if self.other():
                if not self.checkResultsfininsh():
                    continue
                else:
                    print("进入other 判断 再次确认finish")
                    if self.thread_pool.work_queue.is_empty() and self.thread_pool.result_queue.is_empty() \
                            and len(self.results) == 0:
                        print("运行完毕")
                        if self.is_break():
                            print("10 s break")
                            time.sleep(10)
                            break



# -*- coding: utf-8 -*-
# @Time    : 2020/04/30 11:32
# @Author  : suhong
# @FileName: papent.py
# @Software: PyCharm
import logging
import time
import re
import os
from contextlib import closing

import requests
bar_length = 20



class Parent(object):

    def __int__(self):
        self.PDFRoot =""


    def isValidPDF_pathfile(self, pathfile):

        with open(pathfile, mode='rb') as f:
            content = f.read()
        partBegin = content[0:20]
        if partBegin.find(rb'%PDF-1.') < 0:
            print('Error: not find %PDF-1.')
            return False

        idx = content.rfind(rb'%%EOF')
        if idx < 0:
            print('Error: not find %%EOF')
            return False

        partEnd = content[(0 if idx - 100 < 0 else idx - 100): idx + 5]
        if not re.search(rb'startxref\s+\d+\s+%%EOF$', partEnd):
            print('Error: not find startxref')
            return False

        return True

    def Record(self, text,journal):
        name = time.strftime('%Y%m%d', time.localtime(time.time())) + '.txt'
        fileroot1 = os.path.join(self.PDFRoot, journal)
        filepath = os.path.join(fileroot1,name)
        with open(filepath, "a", encoding="utf-8") as f:
            line =text.strip() + '\n'
            f.write(line)

    def makedir(self,journal, years, num,tiele,pdfRoot):
        self.PDFRoot =pdfRoot
        newPdfPath = os.path.join(self.PDFRoot, journal, years, num)
        if not os.path.exists(newPdfPath):
            os.makedirs(newPdfPath)
        tiele = tiele.replace('\\', '_').replace('/', '_').replace(':', '_').replace('>', '_').replace('*', '_').replace('?','_').replace('<', '_').replace('|', '_').replace('"', '_').replace(':', '_')
        pdfFilePath = os.path.join(newPdfPath, tiele + '.pdf')
        if os.path.exists(pdfFilePath):
            filesize = os.path.getsize(pdfFilePath)
            if filesize <= 3072:
                print("%s,pdf存在，大小有问题" % tiele)
                os.remove(pdfFilePath)

            else:
                print('%s, pdf存在' % tiele)
                return 1
        return pdfFilePath

    def formatFloat(self,num):
        # 进度条函数
        return '{:.2f}'.format(num)

    def getPdf(self, pdfurl, title, pdfFilePath,hdrs):
        hashes = None
        try:
            try:
                r = requests.get(pdfurl)
            except:
                print('%s, pdf下载失败' % title)
                return -1
            if r== '':
                print('%s,pdf下载失败' % title)
                return -1
            print("pdf:%s"%pdfFilePath)

            if r.text.find("本篇文章目前没有全文可供下载！")> 1 or r.text.find("提示：您所请求的文件不存在！")  > 1:
                print('%s,pdf无全文' % title)
                return -1
            length = float(len(r.content))
            count = 0
            time1 = time.time()


            with closing(r) as r:
                with open(pdfFilePath, 'wb') as f:

                    for chunk in r.iter_content(chunk_size=512):

                        if chunk:
                            f.write(chunk)
                            count += len(chunk)
                            if time.time() - time1 > 1:
                                print("**********")
                                p = count / length * 100
                                hashes = '▆' * int(p / 100.0 * bar_length)
                                spaces = ' ' * (bar_length - len(hashes))
                                print("\r----%s: |%s| %s%% " % (title, hashes + spaces, self.formatFloat(p)), end="")
                                time.sleep(0.3)
                                time1 = time.time()

            if count == length:
                hashes = '▆' * bar_length
                spaces = ' ' * (bar_length - len(hashes))
                print("\r----%s: |%s| %s%% " % (title, hashes + spaces, "100"), end="")
                print("")
                return 1

            if not Parent.isValidPDF_pathfile(self, pdfFilePath):
                os.remove(pdfFilePath)
                return -1

            return 1
        except Exception as e:
            print(str(e))
            if hashes:
               pass
            logging.info("sava_%s error %s" % (title,str(e)))
            return False

    def post_getpdf(self, data, pdfurl, title, pdfFilePath,hdrs):
        hashes = None
        try:
            try:
                r = requests.post(pdfurl,data=data)
            except:
                print('%s, pdf下载失败' % title)
                return -1
            if r == '':
                print('%s,pdf下载失败' % title)
                return -1
            print("pdf:%s" % pdfFilePath)
            length = float(len(r.content))
            count = 0
            time1 = time.time()

            with closing(r) as r:
                with open(pdfFilePath, 'wb') as f:

                    for chunk in r.iter_content(chunk_size=512):

                        if chunk:
                            f.write(chunk)
                            count += len(chunk)
                            if time.time() - time1 > 1:
                                print("**********")
                                p = count / length * 100
                                hashes = '▆' * int(p / 100.0 * bar_length)
                                spaces = ' ' * (bar_length - len(hashes))
                                print("\r----%s: |%s| %s%% " % (title, hashes + spaces, self.formatFloat(p)), end="")
                                time.sleep(0.3)
                                time1 = time.time()

            if count == length:
                hashes = '▆' * bar_length
                spaces = ' ' * (bar_length - len(hashes))
                print("\r----%s: |%s| %s%% " % (title, hashes + spaces, "100"), end="")
                print("")
                return 1

            if not Parent.isValidPDF_pathfile(self, pdfFilePath):
                os.remove(pdfFilePath)
                return -1

            return 1
        except Exception as e:
            print(str(e))
            if hashes:
                pass
            logging.info("sava_%s error %s" % (title, str(e)))
            return False

    def getPdfbysn(self,sn, pdfurl, title, pdfFilePath,hdrs):
        hashes = None
        try:
            try:
                r = sn.get(pdfurl)
            except:
                print('%s, pdf下载失败' % title)
                return -1
            if r== '':
                print('%s,pdf下载失败' % title)
                return -1
            print("pdf:%s"%pdfFilePath)
            if r.headers['Content-Type'] == 'text/html; charset=utf-8':
                if r.text.find("您所请求的文件不存在")  > 1:
                    print('%s,pdf无全文' % title)
                    return -1
            length = float(len(r.content))
            count = 0
            time1 = time.time()


            with closing(r) as r:
                with open(pdfFilePath, 'wb') as f:

                    for chunk in r.iter_content(chunk_size=512):

                        if chunk:
                            f.write(chunk)
                            count += len(chunk)
                            if time.time() - time1 > 1:
                                print("**********")
                                p = count / length * 100
                                hashes = '▆' * int(p / 100.0 * bar_length)
                                spaces = ' ' * (bar_length - len(hashes))
                                print("\r----%s: |%s| %s%% " % (title, hashes + spaces, self.formatFloat(p)), end="")
                                time.sleep(0.3)
                                time1 = time.time()

            if count == length:
                hashes = '▆' * bar_length
                spaces = ' ' * (bar_length - len(hashes))
                print("\r----%s: |%s| %s%% " % (title, hashes + spaces, "100"), end="")
                print("")
                return 1

            if not Parent.isValidPDF_pathfile(self, pdfFilePath):
                os.remove(pdfFilePath)
                return -1

            return 1
        except Exception as e:
            print(str(e))
            if hashes:
               pass
            logging.info("sava_%s error %s" % (title,str(e)))
            return False


if __name__ == "__main__":
    pass


# -*- coding: utf-8 -*-
# @Time    : 2020/6/16 9:29
# @Author  : suhong
# @File    : db3_2_sizedb3.py
# @Software: PyCharm
"""
获取输入db3文件夹路径，获取db3表名、列名，创建一个db3
往新的db3写入select信息直到指定大小，切换
"""
import sqlite3
import sys
from re_common.baselibrary.utils.basedir import BaseDir

from re_common.baselibrary.utils.basefile import BaseFile


class UniformSizeDb3(object):
    def __init__(self):
        self.table_name = None
        self.column_name = list()
        self.info_list = list()
        self.all_num = 0

    def work(self, file_path=None, out_path=None,size=200 * 1024 * 1024):
        if file_path is None and out_path is None:
            print("path error")
            sys.exit(-1)
        for path in BaseDir.get_dir_all_files(file_path):
            count = 1
            print(path)
            # print(BaseFile.get_file_size(path)/1024/1024)
            conn = sqlite3.connect(path)
            cur = conn.cursor()
            # 获取表名和字段名称
            cur.execute("SELECT name FROM sqlite_master where type='table'")
            row1 = cur.fetchone()
            self.table_name = row1[0]
            cur.execute("PRAGMA table_info([{}])".format(self.table_name))
            row2 = cur.fetchall()
            for row in row2:
                self.column_name.append(row[1])
            column_name = tuple(self.column_name)
            self.column_name.clear()

            # 获取原始db3的创建table语句
            cur.execute("SELECT sql FROM sqlite_master WHERE type='table' AND name = '{}'".format(self.table_name))
            create_table_sql = cur.fetchone()[0]
            insert_sql = "insert into {} {} values ({})".format(self.table_name, column_name, "?," * len(column_name))
            insert_sql = insert_sql.replace("'", "").replace("?,)", "?)")



            out_file = "{}/{}_{}.db3".format(out_path, BaseFile.get_filename_not_extsep(path), count)
            conn_2 = sqlite3.connect(out_file)
            cur_2 = conn_2.cursor()
            cur_2.execute("SELECT name FROM sqlite_master where type='table'")
            name_row = cur_2.fetchone()
            if name_row is None:
                cur_2.execute(create_table_sql)
                conn_2.commit()

            # 查询原表信息 并插入新表
            cur.execute("select * from {}".format(self.table_name))
            while True:
                rows = cur.fetchone()
                if rows:
                    self.info_list.append(rows)

                    if BaseFile.get_file_size(out_file) >= size:
                        count +=1
                        out_file = "{}/{}_{}.db3".format(out_path, BaseFile.get_filename_not_extsep(path), count)
                        conn_2 = sqlite3.connect(out_file)
                        cur_2 = conn_2.cursor()
                        cur_2.execute("SELECT name FROM sqlite_master where type='table'")
                        name_row = cur_2.fetchone()
                        if name_row is None:
                            cur_2.execute(create_table_sql)
                            conn_2.commit()
                        print("outfilepath:{}".format(out_file))

                    if len(self.info_list) >= 10000:
                        self.all_num += len(self.info_list)
                        cur_2.executemany(insert_sql, self.info_list)
                        conn_2.commit()
                        print("do all sql:{}".format(self.all_num))
                        self.info_list.clear()
                else:
                    break
            self.all_num += len(self.info_list)
            cur_2.executemany(insert_sql, self.info_list)
            conn_2.commit()
            cur_2.close()
            print("do all sql:{}".format(self.all_num))
            self.info_list.clear()
            self.all_num = 0


if __name__ == '__main__':
    file_path = r'E:\更新数据\adb3'
    out_path = r'E:\更新数据\test_db3'
    u = UniformSizeDb3()
    u.work(file_path, out_path)



import _io
import linecache
import os
import platform
import random
import re
import shutil
import time
import traceback

from .core.bottomutils import __os_sep__


class BaseFile(_io.TextIOWrapper):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def seek(self, offset: int, whence: int = 0) -> int:
        """
        :param offset: 开始偏移量，也就是代表需要移动偏移的字节数。
        :param whence: 给offset参数一个定义，表示要从哪个位置开始偏移；0
        代表从文件开头开始算起，1代表从当前位置开始算起，2代表从文件末尾算起
        :return: 返回的游标的位置
        """
        super().seek(offset, whence=whence)
        return super().tell()

    def truncate(self, size: int = None) -> int:
        """
        Python 文件 truncate() 方法用于截断文件并返回截断的字节长度。

        指定长度的话，就从文件的开头开始截断指定长度，其余内容删除；
        不指定长度的话，就从文件开头开始截断到当前位置，其余内容删除。
        文件中的数据也会删除
        :param size:
        :return:
        """
        super().truncate(size)
        return super().tell()

    @classmethod
    def read_line(cls, file_path, encoding="utf-8"):
        """
        按行读取文件
        会一次性读取文件生成list  不适合大文件
        :param file_path:
        :param encoding:
        :return:
        """
        try:
            with open(file_path, "r", encoding=encoding) as f:
                for line in f.readlines():
                    yield line.strip()
        except:
            print(traceback.format_exc())

    @classmethod
    def read_file_r_mode_yield(cls, filePath, encoding="utf-8"):
        """
        rb 模式比r 模式更快 但要通过编码格式转换才能使用 对于转移文件使用会更快
        :param filePath:
        :return:
        """
        with open(filePath, "r", encoding=encoding) as f:
            for fLine in f:
                yield fLine.strip()

    @classmethod
    def read_file_rb_mode_yield(cls, filePath):
        """
        rb 方式读取文件
        for line in f文件对象f视为一个迭代器，会自动的采用缓冲IO和内存管理，所以你不必担心大文件
        参数为"rb"时的效率是"r"的6倍 建议rb
        这里说下转str的方法 str(line, encoding='utf-8')
        :param filePath:
        :return: type bytes
        """
        with open(filePath, "rb") as f:
            for fLine in f:
                yield fLine

    @classmethod
    def read_file_rb_block(cls, filePath, BLOCK_SIZE=1024):
        """
        按块读取文件
        :param filePath:
        :param BLOCK_SIZE:
        :return:
        """
        with open(filePath, 'rb') as f:
            while True:
                block = f.read(BLOCK_SIZE)
                if block:
                    yield block
                else:
                    return

    @classmethod
    def read_file_rb(cls, filePath: str) -> bytes:
        assert cls.is_file_exists(filePath), FileNotFoundError("文件不存在%s" % filePath)
        with open(filePath, mode='rb') as f:
            return f.read()

    @classmethod
    def bytes_takeout_utf8bom(cls, content: bytes):
        if content.startswith(b'\xef\xbb\xbf'):  # 去掉 utf8 bom 头
            return content[3:]
        return content

    @classmethod
    def readfile_rb_takeout_utf8bom(cls, cfgFile: str) -> bytes:
        content = cls.read_file_rb(cfgFile)
        content = cls.bytes_takeout_utf8bom(content)
        return content

    @classmethod
    def get_new_path(cls, path, *paths):
        """
        通过当前目录组合新的目录
        :return:
        """
        if platform.system() == "Windows":
            if (re.match("^[A-Za-z]:$", path)):
                path = path + "\\"
        path = os.path.join(path, *paths)
        return path

    @classmethod
    def check_create_dir(cls, path):
        """
        检查和创建目录
        :param path:
        :return:
        """
        if os.path.exists(path):
            return True
        else:
            os.makedirs(path)
            return False

    @classmethod
    def single_write_file(cls, files, value, encoding="utf-8"):
        """
        单次将内容写入文件
        写入值到文件 每次写入会覆盖原来的文件  文件不存在会自动创建
        :param value:
        :param files:
        :return:
        """
        with open(files, 'w', encoding=encoding) as f:
            f.write(value)

    @classmethod
    def single_write_file_lines(cls, files, value_list, encoding="utf-8"):
        """
        单次将内容写入文件
        写入值到文件 每次写入会覆盖原来的文件  文件不存在会自动创建
        :param value:
        :param files:
        :return:
        """
        with open(files, 'w', encoding=encoding) as f:
            f.writelines(value_list)

    @classmethod
    def single_write_wb_file(cls, files, value):
        """
        单次将内容写入文件
        写入值到文件 每次写入会覆盖原来的文件  文件不存在会自动创建
        :param value:
        :param files:
        :return:
        """
        with open(files, 'wb') as f:
            f.write(value)

    @classmethod
    def single_write_ab_file(cls, files, value):
        """
        单次将内容写入文件
        写入值到文件 每次写入会追加原来的文件  文件不存在会自动创建
        :param value:
        :param files:
        :return:
        """
        with open(files, 'ab') as f:
            f.write(value)

    @classmethod
    def get_file_absolute(cls, file):
        """
        请传入 __file__ 作为参数
        获取一个文件的绝对路径
        :param __file__:
        :return:
        """
        return os.path.abspath(file)

    @classmethod
    def single_read_file(cls, filePath, encoding="utf-8", num=-1):
        """
        读取文件中的所有值
        :param filePath:
        :param encoding:
        :param num  读取指定长度 可以用于解决无换行大文件问题 默认-1读取全部
        :return:
        """
        with open(filePath, encoding=encoding) as f:
            return f.read(num)

    @classmethod
    def single_read_rb_file(cls, filePath):
        """
        读取文件中的所有值
        :param filePath:
        :param encoding:
        :return:
        """
        with open(filePath, "rb") as f:
            return f.read()

    @classmethod
    def get_file_row(cls, filePath, linenum):
        return linecache.getline(filePath, linenum)

    @classmethod
    def get_file_line_num(cls, filePath, encoding="utf-8"):
        """
        获取文件行数
        :param filePath:
        :return:
        """
        count = 0
        for index, line in enumerate(open(filePath, 'r', encoding=encoding)):
            count += 1
        return count

    @classmethod
    def single_add_file(cls, filePath, value, encoding="utf-8"):
        """
        追加文件中的所有值
        文件不存在创建
        :param filePath:
        :param encoding:
        :return:
        """
        with open(file=filePath, mode="a", encoding=encoding) as f:
            f.write(value)

    @classmethod
    def add_file_ab_mode(cls, filePath, value):
        """
        追加文件中的所有值
        :param filePath:
        :param encoding:
        :return:
        """
        with open(file=filePath, mode="ab") as f:
            f.write(value)

    @classmethod
    def copy_file_to_file(cls, soufilePath, desfilepath):
        """
        shutil.copy 实现了copy文件
        :param soufilePath: 必须是一个文件
        :param desfilepath: 可以是一个文件 或者目录 如果是文件且文件名不同 可以达到重命名的效果
        当我们想将文件copy成一个无后缀文件 且当前目录已经有一个同名文件夹时 会将文件copy到文件夹中
        而不是一个无后缀文件中 这里要注意在大量文件和文件夹中使用时产生的歧义，且不能定义一个文件与文件夹
        重名
        :return:
        """
        assert cls.is_file(soufilePath), FileNotFoundError(soufilePath + "这不是一个文件的路径")
        # 不存在进行创建目录
        if not os.path.exists(desfilepath):
            # 判断上级目录是否存在
            if not os.path.exists(os.path.split(desfilepath)[0]):
                os.makedirs(os.path.split(desfilepath)[0])
        # copy文件
        shutil.copy(soufilePath, desfilepath)
        # 判断copy后的文件是否存在
        if os.path.isfile(desfilepath):
            return True
        else:
            raise FileExistsError(desfilepath + "不存在，可能被copy到" + desfilepath + "目录中")

    @classmethod
    def get_image_filename_time(cls, filename):
        # 这是从文件名中获取时间  主要用于手机图片的命名格式有效
        li = filename.split(__os_sep__)
        if li[-1].find(__os_sep__) != -1:
            return ''
        if li[-1].find(os.extsep) != -1:
            li2 = li[-1].split(os.extsep)
            li3 = li2[-2].split("_")
            if len(li3) == 1:
                return li3[0]
            return li3[-2] + li3[-1]
        return ''

    @classmethod
    def get_time(cls, filename):
        """
        获取文件时间，这个是获取文件的属性，不利于获取创造时间 因为每一次更改时间都会更新
        :param filename:
        :return:
        """
        ModifiedTime = time.localtime(os.stat(filename).st_mtime)  # 文件访问时间
        y = time.strftime('%Y', ModifiedTime)
        m = time.strftime('%m', ModifiedTime)
        d = time.strftime('%d', ModifiedTime)
        H = time.strftime('%H', ModifiedTime)
        M = time.strftime('%M', ModifiedTime)
        import datetime
        d2 = datetime.datetime(int(y), int(m), int(d), int(H), int(M))
        return d2

    @classmethod
    def secure_filename(cls, filename):
        r"""Pass it a filename and it will return a secure version of it.  This
        filename can then safely be stored on a regular file system and passed
        to :func:`os.path.join`.  The filename returned is an ASCII only string
        for maximum portability.

        On windows systems the function also makes sure that the file is not
        named after one of the special device files.

        经我改变 支持中文

        >>> secure_filename("My cool movie.mov")
        'My_cool_movie.mov'
        >>> secure_filename("../../../etc/passwd")
        'etc_passwd'
        >>> secure_filename(u'i contain cool \xfcml\xe4uts.txt')
        'i_contain_cool_umlauts.txt'

        The function might return an empty filename.  It's your responsibility
        to ensure that the filename is unique and that you generate random
        filename if the function returned an empty one.

        .. versionadded:: 0.5

        :param filename: the filename to secure
        """
        if isinstance(filename, text_type):
            # 此模块提供对Unicode字符数据库的访问，该字符数据库为所有Unicode字符定义字符属性。该数据库中的数据基于UnicodeData.txt
            # 可从ftp://ftp.unicode.org/公开获得的文件版本5.2.0 。
            from unicodedata import normalize
            # ignore参数会忽略无法编码的字符
            # normalize 为标准化底层编码 http://python3-cookbook.readthedocs.io/zh_CN/latest/c02/p09_normalize_unicode_text_to_regexp.html
            filename = normalize('NFKD', filename).encode('utf-8')
            if not PY2:
                filename = filename.decode('utf-8')
        for sep in os.path.sep, os.path.altsep:
            if sep:
                # 替换用后面的替换前面的
                filename = filename.replace(sep, ' ')
        filename = str(re.sub(':', '', '_'.join(
            filename.split()))).strip('._')

        # on nt a couple of special files are present in each folder.  We
        # have to ensure that the target file is not such a filename.  In
        # this case we prepend an underline
        if os.name == 'nt' and filename and \
                filename.split('.')[0].upper() in _windows_device_files:
            filename = '_' + filename

        return filename

    @classmethod
    def get_file_ext_name(cls, filename, double_ext=False):
        """
        获取文件后缀
        os.extsep = "."
        :param filename: 传入文件名或者带路径的文件
        :param double_ext:
        :return:
        """
        li = filename.split(os.extsep)  # 以.分割
        if len(li) <= 1:  # 如果分割后只有一个  那么这没有文件后缀
            return ''
        else:
            if li[-1].find(__os_sep__) != -1:  # 如果在最后一个中发现分割符 也没有后缀
                return ''
        if double_ext:
            if len(li) > 2:  # 有可能后缀有多个组成 这里我们最多返回两个字符串以点链接形成的后缀
                if li[-2].find(__os_sep__) == -1:
                    return '%s.%s' % (li[-2], li[-1])
        return li[-1]

    @classmethod
    def get_filename_not_extsep(cls, filename):
        """
        获取文件名  去除后缀和路径
        :param filename:
        :return:
        """
        filename = cls.get_file_name(filename)
        return os.extsep.join(filename.split(os.extsep)[:-1])

    @classmethod
    def get_file_name(cls, filename):
        """
        得到文件名  去掉路径
        :param filename:
        :return:
        """
        # assert cls.is_file(filename), FileNotFoundError("传入的文件路径不正确")
        return os.path.split(filename)[-1]
        # 旧的方式 废弃
        # return filename.split(__os_sep__)[-1]

    @classmethod
    def new_title(cls, title):
        """
        替换掉标题里的特殊符号 方便保存
        :param title:
        :return:
        """
        err_str = r"[\/\\\:\*\?\"\<\>\|]"
        new_title = re.sub(err_str, "_", title)
        return new_title

    @classmethod
    def is_file_exists(cls, pathdir):
        """
        目录是否存在
        :param sPth:
        :return:
        """
        if os.path.exists(pathdir) and cls.is_file(pathdir):
            return True
        else:
            return False

    @classmethod
    def get_file_size(cls, filePath):
        assert os.path.isfile(filePath), FileNotFoundError("你给出的路径的文件不存在")
        return os.path.getsize(filePath)

    @classmethod
    def is_file(cls, spath):
        """
        判断传入的路径是否是一个文件
        :param spath:
        :return:
        """
        if os.path.isfile(spath):
            return True
        return False

    @classmethod
    def remove_file(cls, filepath):
        """
        删除文件
        :param filepath:
        :return:
        """
        os.remove(filepath)

    @classmethod
    def rename_file(cls, filepath, sourpath):
        os.rename(filepath, sourpath)

    @classmethod
    def get_create_time(cls, filepath):
        # 创建时间
        return os.path.getctime(filepath)

    @classmethod
    def get_update_time(cls, filepath):
        # 修改时间
        return os.path.getmtime(filepath)

    @classmethod
    def change_file(cls, filepath, sign='_', ending='big_json', size=2 * 1024 * 1024 * 1024):
        """
        :param filepath: 传入验证文件
        :param size: 文件大小限制(字节比较)(默认为2GB)
        :param sign: 文件切换后缀的标志 默认为"_"
        :param ending: 后缀名
        :return: 一个新的文件名
        注意 首次传入的文件名不能包含sign标志位
        """
        from re_common.baselibrary.utils.basedir import BaseDir
        uppath = BaseDir.get_upper_dir(filepath, -1)
        if not cls.is_file_exists(filepath):
            return filepath
        else:
            if cls.get_file_size(filepath) >= size:
                import random
                if sign in cls.get_filename_not_extsep(filepath):
                    path_list = cls.get_file_name(filepath).split(sign)
                    filename = path_list[0] + sign + repr(random.randrange(11111, 99999)) + "." + ending
                else:
                    filename = cls.get_file_name(filepath)
                    filename = filename.replace(("." + cls.get_file_ext_name(filepath)),
                                                (sign + repr(random.randrange(11111, 99999)) + "." + ending))
                path = cls.get_new_path(uppath, filename)
                if cls.is_file_exists(path):
                    return cls.change_file(path, sign=sign, ending=ending,size=size)
                return path
            else:
                return filepath


    @classmethod
    def get_new_filename(cls, filepath, sign='_', ending='big_json.gz'):
        if not cls.is_file_exists(filepath):
            return filepath
        else:
            from re_common.baselibrary.utils.basedir import BaseDir
            uppath = BaseDir.get_upper_dir(filepath, -1)
            if sign in cls.get_filename_not_extsep(filepath):
                path_list = cls.get_file_name(filepath).split(sign)
                filename = path_list[0] + sign + repr(random.randrange(11111, 99999)) + "." + ending
            else:
                filename = cls.get_file_name(filepath)
                filename = filename.replace(("." + cls.get_file_ext_name(filepath)),
                                            (sign + repr(random.randrange(11111, 99999)) + "." + ending))
            path = cls.get_new_path(uppath, filename)
            if cls.is_file_exists(path):
                return cls.change_file(path, sign=sign, ending=ending)
            return path

    @classmethod
    def file_is_update_days(cls, filepath, days=3):
        """
        判断文件是否在3天内 有更新
        True 3天内有更新 否则更新在3天以前
        :param days:
        :return:
        """
        if BaseFile.is_file_exists(filepath):
            ctime = os.path.getctime(filepath)  # 创建时间
            mtime = os.path.getmtime(filepath)  # 修改时间
            if ((time.time() - ctime) / 3600 / 24 < days) or ((time.time() - mtime) / 3600 / 24 < days):
                return True
            else:
                return False
        else:
            return False

    @classmethod
    def copy_file(cls, file, targetdir):
        """

        :param file:
        :param destdir: 目标目录
        :return:
        """
        assert cls.is_file_exists(file)
        shutil.copyfile(file, targetdir)



class BaseAbs(object):
    """
    抽象工厂模式，方便管理和引用 目前主要将配置文件和sql相关的库引入了抽象工厂模式中
    """
    @staticmethod
    def get_sql_factory():
        """
        sql相关的工厂
        :return:
        """
        from .database.sql_factory import SqlFactory
        return SqlFactory()

    @staticmethod
    def get_config_factory():
        """
        配置文件相关的工厂
        :return:
        """
        from .readconfig.config_factory import ConfigFactory
        return ConfigFactory()






import os
import threading
import time
from abc import ABC, abstractmethod

from re_common.baselibrary.mthread.mythreading import ThreadPoolManger, ThreadInfo
from re_common.baselibrary.utils.ringlist import RingList
from re_common.facade.loggerfacade import get_streamlogger


class especialThreadName(object):

    def __init__(self):
        self.taskthreadname = "my_threading_taskthread_1"
        self.proxythreadname = "my_threading_proxythread_1"
        self.dealresultthreadname = "my_threading_dealresult_1"

    def list_name(self):
        return [self.taskthreadname, self.proxythreadname, self.dealresultthreadname]


class MThreadingRun(ABC):
    def __init__(self, num, logger=None):
        if logger:
            self.logger = logger
        else:
            self.logger = get_streamlogger()
        self.etn = especialThreadName()
        # 线程数
        self.threadingnum = num
        # 代理列表
        self.list_proxy = RingList()
        # 线程池
        self.thread_pool = ThreadPoolManger(self.threadingnum, self.logger)
        self.thread_pool.set_callback(self.thread_pool_hook)
        # 结果集
        self.results = []
        # 线程方法
        self.func = self.fun
        # 结果数
        self.resultnum = 0
        # 工作线程数
        self.jobnum = 0
        # 结果集被处理标志  默认被处理 是为了兼容之前的代码不去改动
        self.dealresultstatus = True
        # 結果到达该数量后处理结果 默认及时处理
        self.dealresultsnum = 0
        # 代理设置时间
        self.proxytime = 0
        # 标识event set之后是否对任务进行了设置
        # self.eventbool = False
        self.modle = 1
        # 在任务和处理结果时event信号的状态
        self.result_event_status = True
        self.task_event_status = True
        self.thread_run_lock = threading.Lock()

        # 默认每次处理的结果数
        self.once_result_num = 100

        # 全局使用特殊的单词
        self.BREAK = "break"

        # 进程号
        if hasattr(os, 'getpid'):
            self.pid = os.getpid()
        else:
            self.pid = None

    def set_is_restart(self, name, is_restart):
        self.thread_pool.especial_thread_pool_dicts[name].set_is_restart(is_restart)

    @abstractmethod
    def setProxy(self, proxysList=None):
        """
        将代理加入到循环队列中 self.list_proxy
        :param proxysList:
        :return:
        """
        pass

    @abstractmethod
    def fun(self, threadval, *args, **kwargs):
        pass

    @abstractmethod
    def thread_pool_hook(self, threadinfo: ThreadInfo) -> dict:
        """
        钩子函数 可以被重写
        主要重写里面的dicts部分
        :return:
        """
        return {}

    @abstractmethod
    def setTask(self, results=None, *args, **kwargs):
        # self.thread_pool.event.set()  # 自动释放信号
        # 当设置任务和处理结果使用同一个连接时尽量使用event信号保证不同时执行sql在一个执行未返回时
        # 请查询较大数据时主动释放
        pass

    @abstractmethod
    def getTask(self, *args, **kwargs):
        """
        主要用于mysql的请求
        :param args:
        :param kwargs:
        :return:
        """
        pass

    @abstractmethod
    def dealresult(self, *args, **kwargs):
        """
        如果没有在这里处理 请将 self.dealresultstatus = False这样不会丢数据
        :param args:
        :param kwargs:
        :return:
        """
        pass

    def set_task(self, *args, **kwargs):
        """
        设置任务，在没有event限制的情况下将两个步骤写在一起
        :param args:
        :param kwargs:
        :return:
        """
        while True:
            results = self.getTask(*args, **kwargs)
            result = self.setTask(results, *args, **kwargs)
            if result == self.BREAK:
                break

    def deal_results(self, *args, **kwargs):
        """
        该函数用于没有信号的机制里
        使用信号主要是因为多线程无法同时使用一个链接
        有可能出现错误
        :param args:
        :param kwargs:
        :return:
        """
        while True:
            # 处理结果标识
            self.dealresultstatus = True
            # 从结果队列获取结果到results
            self.getreustlFromQueue()
            if len(
                    self.results) > self.dealresultsnum or self.thread_pool.work_queue.is_empty() or not self.thread_pool.thread_queue:
                if len(self.results) > 0 or not self.thread_pool.thread_queue.is_empty():
                    # 处理结果
                    self.dealresult()
                    if self.dealresultstatus:
                        # 处理完结果后要清理
                        self.results.clear()
                else:
                    time.sleep(10)
            else:
                time.sleep(3)

    def setfunc(self, func):
        # 设置线程方法
        self.func = func

    def add_job(self, func, *args, **kwargs):
        self.jobnum += 1
        self.thread_pool.add_job(func, *args, **kwargs)

    def getreustlFromQueue(self):
        """
        从结果队列获取结果到results
        默认取100
        :return:
        """
        once_result_num = 0
        while not self.thread_pool.result_queue.is_empty():
            self.resultnum += 1
            once_result_num += 1
            if once_result_num > self.once_result_num:
                return
            result = self.thread_pool.result_queue.get()
            self.results.append(result)
            self.thread_pool.result_queue.task_done()

    def checkResultsfininsh(self, *args, **kwargs):
        """
        该函数用于处理运行到最后时结果不足100的情况
        :return:
        """
        if self.thread_pool.work_queue.is_empty():
            t1 = len(self.results)
            self.logger.info("self.results len is %s " % str(t1))
            if t1 != 0:
                return False
            else:
                return True

    def other(self):
        self.logger.info("工作队列任务量为{},结果队列任务量为{}".format(self.thread_pool.work_queue.get_size(),
                                                        self.thread_pool.result_queue.get_size()))
        # 通过工作队列和结果队列观察是否结束
        if self.thread_pool.checkThreadRunFinish():
            self.logger.info("初次判断任务已经结束，各个队列为空")
            return True
        else:
            return False

    def check_especial_thread(self):
        task = self.set_task
        proxy = self.setProxy
        result = self.deal_results

        nowThreadsName = self.thread_pool.get_now_thread()
        for name in list(self.thread_pool.especial_thread_pool_dicts.keys()):
            thread = self.thread_pool.especial_thread_pool_dicts[name].get_thread()
            # 如果线程字典为空 代表已被删除
            if name in nowThreadsName and thread.is_alive():
                # print(name + ": is run")
                pass  # 当前某线程名包含在初始化线程组中，可以认为线程仍在运行
            else:
                self.logger.info("name is :" + name + "; 没有在线程中")
                if name in self.etn.list_name():
                    if name == self.etn.taskthreadname:
                        taskin = task
                    elif name == self.etn.proxythreadname:
                        taskin = proxy
                    elif name == self.etn.dealresultthreadname:
                        taskin = result
                    else:
                        raise Exception("没有对应的任务，请检查")
                    is_start = False
                    if name in self.thread_pool.especial_thread_pool_dicts:
                        threadinfo = self.thread_pool.especial_thread_pool_dicts[name]
                        if threadinfo.get_thread().is_alive():
                            is_start = True
                        if not threadinfo.get_is_restart():
                            is_start = True
                    if not is_start:
                        self.logger.info('重启中:' + name)
                        args = self.thread_pool.especial_thread_pool_dicts[name].get_args()
                        kwargs = self.thread_pool.especial_thread_pool_dicts[name].get_kwargs()
                        thread = self.thread_pool.set_add_especial_thread(taskin, name, *args, **kwargs)
                        thread.start()

    def start_especial_thread(self):
        # 开启一个线程设置任务
        self.thread_pool.set_add_especial_thread(self.set_task, self.etn.taskthreadname)
        self.thread_pool.set_add_especial_thread(self.setProxy, self.etn.proxythreadname)
        self.thread_pool.set_add_especial_thread(self.deal_results, self.etn.dealresultthreadname)
        self.thread_pool.especial_start()

    def is_break(self):
        return False

    def run(self):
        self.start_especial_thread()
        while True:
            time.sleep(3)
            self.thread_pool.checkThread()
            self.check_especial_thread()
            if self.other():
                if not self.checkResultsfininsh():
                    continue
                else:
                    print("进入other 判断 再次确认finish")
                    if self.thread_pool.work_queue.is_empty() and self.thread_pool.result_queue.is_empty() \
                            and len(self.results) == 0:
                        print("运行完毕")
                        if self.is_break():
                            print("10 s break")
                            time.sleep(10)
                            break
                        else:
                            time.sleep(60)


class MThreadingRun2(ABC):
    """
    多线程的封装
    """
    def __init__(self, num, logger=None):
        # 日志模块，如果无会自定义一个只打印不输入到文件的logger
        if logger:
            self.logger = logger
        else:
            self.logger = get_streamlogger()
        # 特殊线程的类，对名称进行了统一
        self.etn = especialThreadName()
        # 工作线程数，不包含特殊线程
        self.threadingnum = num
        # 代理列表 RingList是一个环形列表
        self.list_proxy = RingList()
        # 工作线程池
        self.thread_pool = ThreadPoolManger(self.threadingnum, self.logger)
        # 钩子函数，将thread_pool的钩子函数暴露到这一层，方便使用
        self.thread_pool.set_callback(self.thread_pool_hook)
        # 线程方法
        self.func = self.fun
        # 工作量的累计，每一次addjob都会累计一个数
        self.jobnum = 0
        # 代理设置时间
        # self.proxytime = 0

        # self.modle = 1
        # 线程锁，外部使用
        self.thread_run_lock = threading.Lock()

        # 全局使用特殊的单词
        self.BREAK = "break"

        # 进程号
        if hasattr(os, 'getpid'):
            self.pid = os.getpid()
        else:
            self.pid = None

    def set_is_restart(self, name, is_restart):
        """
        特殊线程的重启设置，默认都会重启，设置False不允许重启
        :param name:
        :param is_restart:
        :return:
        """
        self.thread_pool.especial_thread_pool_dicts[name].set_is_restart(is_restart)

    @abstractmethod
    def setProxy(self, proxysList=None):
        """
        将代理加入到循环队列中 self.list_proxy，或者自己管理代理
        :param proxysList:
        :return:
        """
        pass

    @abstractmethod
    def fun(self, threadval, *args, **kwargs):
        """
        运行的方法
        :param threadval:
        :param args:
        :param kwargs:
        :return:
        """
        pass

    @abstractmethod
    def thread_pool_hook(self, threadinfo: ThreadInfo) -> dict:
        """
        钩子函数 可以被重写
        主要重写里面的ThreadInfo的信息
        :return:
        """
        return {}

    @abstractmethod
    def set_task(self, *args, **kwargs):
        """
        设置任务，这里调用addjob
        :param args:
        :param kwargs:
        :return:
        """
        pass

    def deal_results(self, *args, **kwargs):
        """
        处理结果
        :param args:
        :param kwargs:
        :return:
        """
        self.dealresults(self.thread_pool.result_queue, *args, **kwargs)

    @abstractmethod
    def dealresults(self, result_queue, *args, **kwargs):
        """
        每次调用result_queue.get() 后需要调用result_queue.task_done()进行确认
        result = result_queue.get()
        ...
        result_queue.task_done()
        :param result_queue:
        :return:
        """
        pass

    def setfunc(self, func):
        # 设置线程方法,默认是self.fun
        self.func = func

    def add_job(self, func, *args, **kwargs):
        """
        :param func: 运行的方法，一般使用self.func
        :param args: 元组参数
        :param kwargs: 键值对参数
        """
        self.jobnum += 1
        self.thread_pool.add_job(func, *args, **kwargs)

    def other(self):
        """
        其他的一些判断
        :return:
        """
        self.logger.info("工作队列任务量为{},结果队列任务量为{}".format(self.thread_pool.work_queue.get_size(),
                                                        self.thread_pool.result_queue.get_size()))
        # 通过工作队列和结果队列观察是否结束
        if self.thread_pool.checkThreadRunFinish():
            self.logger.info("初次判断任务已经结束，各个队列为空")
            return True
        else:
            return False

    def check_especial_thread(self):
        """
        检查特殊线程是否挂掉
        :return:
        """
        task = self.set_task
        proxy = self.setProxy
        result = self.deal_results

        nowThreadsName = self.thread_pool.get_now_thread()
        for name in list(self.thread_pool.especial_thread_pool_dicts.keys()):
            thread = self.thread_pool.especial_thread_pool_dicts[name].get_thread()
            # 如果线程字典为空 代表已被删除
            if name in nowThreadsName and thread.is_alive():
                # print(name + ": is run")
                pass  # 当前某线程名包含在初始化线程组中，可以认为线程仍在运行
            else:
                self.logger.info("name is :" + name + "; 没有在线程中")
                if name in self.etn.list_name():
                    if name == self.etn.taskthreadname:
                        taskin = task
                    elif name == self.etn.proxythreadname:
                        taskin = proxy
                    elif name == self.etn.dealresultthreadname:
                        taskin = result
                    else:
                        raise Exception("没有对应的任务，请检查")
                    is_start = False
                    if name in self.thread_pool.especial_thread_pool_dicts:
                        threadinfo = self.thread_pool.especial_thread_pool_dicts[name]
                        if threadinfo.get_thread().is_alive():
                            is_start = True
                        if not threadinfo.get_is_restart():
                            is_start = True
                    if not is_start:
                        self.logger.info('重启中:' + name)
                        args = self.thread_pool.especial_thread_pool_dicts[name].get_args()
                        kwargs = self.thread_pool.especial_thread_pool_dicts[name].get_kwargs()
                        thread = self.thread_pool.set_add_especial_thread(taskin, name, *args, **kwargs)
                        thread.start()

    def start_especial_thread(self):
        # 开启特殊线程设置任务
        self.thread_pool.set_add_especial_thread(self.set_task, self.etn.taskthreadname)
        self.thread_pool.set_add_especial_thread(self.setProxy, self.etn.proxythreadname)
        self.thread_pool.set_add_especial_thread(self.deal_results, self.etn.dealresultthreadname)
        self.thread_pool.especial_start()

    def is_break(self):
        """
        运行完毕是否退出多线程
        :return:
        """
        return False

    # 在主线程添加的钩子函数，每次循环都会调用到
    def run_hook_everytime(self):
        time.sleep(3)

    # 运行完毕的钩子函数
    def run_hook_result(self):
        pass

    def run(self):
        """
        主进程代码，线程设置为主进程结束所有线程都会结束
        所以任务完成之前主进程不应该结束
        :return:
        """
        self.start_especial_thread()
        while True:
            self.run_hook_everytime()
            self.thread_pool.checkThread()
            self.check_especial_thread()
            if self.other():
                print("进入other 判断 再次确认finish")
                if self.thread_pool.work_queue.is_empty() and self.thread_pool.result_queue.is_empty():
                    print("运行完毕")
                    self.run_hook_result()
                    if self.is_break():
                        print("10 s break")
                        time.sleep(10)
                        break
                    else:
                        time.sleep(60)



import datetime
import time
from calendar import monthrange, Calendar


class BaseTime(object):

    def __init__(self):
        self.mcal = MCalendar()

    def date_format(self, datestring, sourceformat='%b %d, %Y', tarformat="%Y%m%d"):
        """
        特定字符串转时间 转 string
        MAY 24, 2017  ==》 20170524
        :param datestring: 时间字符串
        :param sourceformat: 源格式 源格式需要与传入的字符串格式一致
        :param tarformat: 目标格式
        :return: 一个目标格式的string
        """
        """
        MAY 24, 2017  ==》 20170524
        :param datestring:
        :return:
        """
        return datetime.datetime.strftime(datetime.datetime.strptime(datestring, sourceformat), tarformat)

    def get_now_datetime(self):
        """
        获取现在的时间
        :return: {
        type: datetime.datetime
        cmd python: datetime.datetime(2019, 1, 28, 17, 19, 5, 614144)
        print Strings: 2019-01-28 17:15:47.671929
        }
        """
        return datetime.datetime.now()

    def string_to_datetime(self, str, format="%Y-%m-%d %H:%M:%S.%f") -> datetime.datetime:
        """
        string 转 datetime
        :param str: 2017-01-01 11:10:45.1234
        :param format:  格式 必须和str的格式相对相应
        :return:{
        type: datetime.datetime
        cmd python: datetime.datetime(2019, 1, 28, 17, 19, 5, 614144)
        print Strings: 2019-01-28 17:15:47.671929
        }
        """
        return datetime.datetime.strptime(str, format)

    def datetime_to_string(self, date, format="%Y.%m.%d"):
        """
        :param date: 输入类型 datetime.datetime
        :return: 2017.06.01
        """
        return date.strftime(format)

    def date_to_datetime(self, date):
        """
        由于datetime与date不能直接比较  所以需要转换一下
        :return:
        """
        return datetime.datetime.combine(date, datetime.datetime.min.time())

    def get_diff(self, datetime):
        """
        传入一个时间标准的str 求差秒
        :param datetime: {type: datetime.datetime}
        :return: string 秒
        """
        return (self.get_now_datetime() - self.string_to_datetime(datetime)).seconds

    def get_time(self):
        """
        返回当前时间的时间戳
        1546914500.7278442
        :return:
        """
        return time.time()

    def get_weeks_after(self, mdatetime, weeks):
        """
        周
        :param days:
        :return: datetime
        """
        return mdatetime + datetime.timedelta(weeks=weeks)

    def get_weeks_before(self, mdatetime, weeks):
        """
        周
        :param days:
        :return: datetime
        """
        return mdatetime - datetime.timedelta(weeks=weeks)

    def get_day_after(self, mdatetime, days):
        """
        获取几天前的时间 并获取strings 可指定时间格式
        :param days:
        :return: datetime
        """
        return mdatetime + datetime.timedelta(days=days)

    def get_day_before(self, mdatetime, days):
        """
        获取几天前的时间 并获取strings 可指定时间格式
        :param days:
        :return: datetime
        """
        return mdatetime - datetime.timedelta(days=days)

    def get_hours_after(self, mdatetime, hours):
        """
        小时
        :param days:
        :return: datetime
        """
        return mdatetime + datetime.timedelta(hours=hours)

    def get_hours_before(self, mdatetime, hours):
        """
        小时
        :param days:
        :return: datetime
        """
        return mdatetime - datetime.timedelta(hours=hours)

    def get_minutes_after(self, mdatetime, minutes):
        """
        分
        :param days:
        :return: datetime
        """
        return mdatetime + datetime.timedelta(minutes=minutes)

    def get_minutes_before(self, mdatetime, minutes):
        """
        分
        :param days:
        :return: datetime
        """
        return mdatetime - datetime.timedelta(minutes=minutes)

    def get_seconds_after(self, mdatetime, seconds):
        """
        秒
        :param days:
        :return: datetime
        """
        return mdatetime + datetime.timedelta(seconds=seconds)

    def get_seconds_before(self, mdatetime, seconds):
        """
        秒
        :param days:
        :return: datetime
        """
        return mdatetime - datetime.timedelta(seconds=seconds)

    def get_milliseconds_after(self, mdatetime, milliseconds):
        """
        毫秒
        :param days:
        :return: datetime
        """
        return mdatetime + datetime.timedelta(milliseconds=milliseconds)

    def get_milliseconds_before(self, mdatetime, milliseconds):
        """
        毫秒
        :param days:
        :return: datetime
        """
        return mdatetime - datetime.timedelta(milliseconds=milliseconds)

    def get_microseconds_after(self, mdatetime, microseconds):
        """
         微秒
        :param days:
        :return: datetime
        """
        return mdatetime + datetime.timedelta(microseconds=microseconds)

    def get_microseconds_before(self, mdatetime, microseconds):
        """
        微秒
        :param days:
        :return: datetime
        """
        return mdatetime - datetime.timedelta(microseconds=microseconds)

    def get_next_month(self, mdatetime, day=1):
        """
        获取下一月某一天
        :param datestring: 输入格式 20170524
        :return:  返回类型 datetime.datetime  格式  2019-01-01 00:00:00
        """
        return (mdatetime.replace(day=1) + datetime.timedelta(33)).replace(day=day)

    def get_befor_month(self, mdatetime, day=1):
        """
        获取前一月的某一天
        :param datestring: 输入格式 20170524
        :return:  返回类型 datetime.datetime  格式  2019-01-01 00:00:00
        """
        return (mdatetime.replace(day=1) - datetime.timedelta(10)).replace(day=day)

    def get_today_date_strings(self):
        """
        获取当前日期级时间字符串
        如 20180810
        :return: string
        """
        return self.datetime_to_string(self.get_now_datetime(), '%Y%m%d')

    def get_beijin_date_strins(self, format="%Y%m%d%H%M%S"):
        """
        获取北京时间string
        :return:
        """
        update_time = self.datetime_to_string(
            datetime.datetime.utcnow() + datetime.timedelta(hours=8), format)
        return update_time

    def get_moths_day(self):
        pass


class MCalendar(object):
    def __init__(self):
        self.c = Calendar()

    def monthrange(self, year: int, moths: int):
        """
        获取一个月的迭代
        :return: tuple (0,31)
        """
        return monthrange(year, moths)

    def itermonthdates(self, year: int, moths: int):
        """
        可以迭代某個月，但前后会有其他月份的日期
        :param year:
        :param moths:
        :return: generator object Calendar.itermonthdates 返回一個生成器
        """
        return self.c.itermonthdates(year, moths)

    def date_iter(self, year, month):
        """
        返回某个月的所有天
        for d in date_iter(2019, 12):
            print(d)
        :param year:
        :param month:
        :return:
        """
        for i in range(1, monthrange(year, month)[1] + 1):
            yield datetime.date(year, month, i)


for i in MCalendar().date_iter(2020,1):
    print(type(i))


# 延迟导入　主要是为了使用时不去安装多余的
from re_common.baselibrary import IniConfig


def get_TomlConfig():
    from re_common.baselibrary.readconfig.toml_config import TomlConfig
    return TomlConfig


def get_streamlogger():
    from re_common.baselibrary import MLogger
    return MLogger().streamlogger



# -*- coding: utf-8 -*-
# @Time    : 2020/4/29 10:12
# @Author  : suhong
# @File    : chinjmap.py
# @Software: PyCharm

# 1.环境科学学报

import sys

import re
import requests
from bs4 import BeautifulSoup
from parsel import Selector
import  parent

hdrs = {"User-Agent":"Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/75.0.3770.100 Safari/537.36",
        "Referer": "http://www.chinjmap.com/ch/reader/issue_list.aspx",
        }

Parent = parent.Parent()

class Chinjmap(object):

    def __init__(self):
        self.journal = '中国现代应用药学'
        self.base_url = "http://www.chinjmap.com/ch/reader/"
        self.ero = 0
        self.suc = 0
        self.cnt = 0
        self.all_num = 0

    def getpdf(self, years, num, pdfRoot):
        years = str(years)
        num = str(num)
        url  = "http://www.chinjmap.com/ch/reader/issue_list.aspx?year_id={}&quarter_id={}".format(years,num)
        print(url)
        try:
            r = requests.get(url, headers=hdrs, timeout=60)
        except:

            print('---------------访问失败--------------')
            return False
        line = 'No.' + str(num)
        print(line)
        html = Selector(r.text)
        # 封面
        fengmian = html.xpath("//p[@align='center']/a/@href").extract_first("")
        if fengmian != "":
            f_url = "http://www.chinjmap.com" + fengmian
            title = "{}封面".format(line)
            pdfFilePath = Parent.makedir(self.journal, years, str(num), title, pdfRoot)
            fig = Parent.getPdf(f_url, title, pdfFilePath, hdrs)
            if fig == 1:
                print('%s,pdf下载成功' % title)
        a_list = html.xpath("//a[contains(@href,'create_pdf')]/@href").extract()
        self.all_num = len(a_list)
        for i,item in enumerate(a_list):
            title = str(i+1)
            pdfurl = self.base_url + item
            print('-----------开始下载-------------: %s' % title)
            pdfFilePath = Parent.makedir(self.journal, years, str(num), title, pdfRoot)
            if pdfFilePath == 1:
                self.suc += 1
                continue
            fig = Parent.getPdf(pdfurl, title, pdfFilePath, hdrs)
            if fig == -1:
                self.ero += 1
                line = 'journal: %s, years: %s, issue: %s: %s 下载失败' % (
                    self.journal, years, num, title.strip())
                line = line + '\n'
                Parent.Record(line, self.journal)
                # print(line)
            if fig == 1:
                print('%s,pdf下载成功' % title)
                self.suc += 1
                line = 'journal: %s, years: %s, issue: %s。共有文章 %d；下载PDF %d；失败 %d' % (
                    self.journal, years, num, self.all_num, self.suc, self.ero)
                line = line + '\n'
                Parent.Record(line, self.journal)
        Parent.Record("\n", self.journal)
        return True

# if __name__ == '__main__':
#     c = Chinjmap()
#     c.getpdf("2020",'4','./')


import asyncio
import json
import zlib

import aiofiles
import aiohttp
import ujson as ujson
from yarl import URL


class BaseAiohttp():

    def __init__(self):
        self.session = aiohttp.ClientSession()
        # 用于传递get的参数
        # params = {'key1': 'value1', 'key2': 'value2'}
        # params = [('key', 'value1'), ('key', 'value2')] 同建多值问题
        # params='key=value+1' 您也可以将str内容作为参数传递，但要注意–内容不是由库编码的。
        # 请注意，+未编码
        self.params = {}
        # post json
        self.json = {}
        # post表单数据 {'key1': 'value1', 'key2': 'value2'}
        self.payload = {}
        # 如果您要发送未经格式编码的数据，可以通过传递a bytes而不是a来完成dict。
        # 这些数据将直接发布，并且默认将内容类型设置为“ application / octet-stream”：
        # =b'\x00Binary-data\x00'
        # 要发送具有适当内容类型的文本，只需使用text属性 data='Тест'
        # 要上传多部分编码的文件，请执行以下操作：
        # files = {'file': open('report.xls', 'rb')}
        # 您可以设置filename和content_type：
        # data = FormData()
        # data.add_field('file',
        #                open('report.xls', 'rb'),
        #                filename='report.xls',
        #                content_type='application/vnd.ms-excel')
        # 如果将文件对象作为数据参数传递，则aiohttp会将其自动流式传输到服务器。
        # 检查StreamReader 支持的格式信息。
        self.data = b''
        # headers可以是单个请求的，也可以是会话的
        self.headers = {}
        # {'cookies_are': 'working'}
        self.cookies = {}

    async def file_sender(self, file_name=None, size=64):
        """
        async with session.post('http://httpbin.org/post',
                        data=file_sender(file_name='huge_file')) as resp:
            print(await resp.text())
        :param file_name:
        :param size:
        :return:
        """
        async with aiofiles.open(file_name, 'rb') as f:
            chunk = await f.read(size * 1024)
            while chunk:
                yield chunk
                chunk = await f.read(size * 1024)

    def get_and_send(self):
        """
        从版本3.1开始不推荐使用：aiohttp仍然支持aiohttp.streamer装饰器，
        但是不推荐使用此方法，而支持异步生成器，如上所示。
        Python 3.5没有对异步生成器的本机支持，请使用 async_generator库作为解决方法。
        因为该content属性是 StreamReader（提供异步迭代器协议），所以您可以将获取和发布请求链接在一起：
        resp = await session.get('http://python.org')
        await session.post('http://httpbin.org/post',
                           data=resp.content)
        :return:
        """
        pass

    def web_sockets(self, url):
        """

        :return:
        """
        async with self.session.ws_connect(url) as ws:
            async for msg in ws:
                if msg.type == aiohttp.WSMsgType.TEXT:
                    if msg.data == 'close cmd':
                        await ws.close()
                        break
                    else:
                        await ws.send_str(msg.data + '/answer')
                elif msg.type == aiohttp.WSMsgType.ERROR:
                    break

    def data_upfile(self, file):
        """

        :param file:
        :return:
        """
        with open(file, 'rb') as f:
            await self.session.post('http://httpbin.org/post', data=f)

    def init_session(self, json_serialize=json.dumps, headers=None,
                     cookies=None, cookie_jar=None, connector=None):
        """

        :param json_serialize: 默认使用json.dumps 这里可以使用 ujson.dumps
        ujson库比标准库快，json但是有点不兼容。
        :param headers:
        :param cookies:
        :param cookie_jar:默认情况下ClientSession使用的严格版本 aiohttp.CookieJar。RFC 2109明确禁止cookie接受具有IP地址而不是DNS名称的URL（例如 http://127.0.0.1:80/cookie）。
        很好，但有时为了进行测试，我们需要启用对此类Cookie的支持。应该通过将unsafe = True传递给 aiohttp.CookieJar构造函数来完成：
        jar = aiohttp.CookieJar(unsafe=True)
        jar = aiohttp.DummyCookieJar()
        :return:
        """

        self.session = aiohttp.ClientSession(json_serialize=json_serialize, headers=headers,
                                             cookies=cookies, cookie_jar=cookie_jar,
                                             connector=connector)

    def deflate_zip(self, data, headers):
        """

        :return:
        """
        headers['Content-Encoding'] = 'deflate'
        return zlib.compress(data), headers

    async def on_request_start(self,
                               session, trace_config_ctx, params):
        print("Starting request")

    async def on_request_end(self, session, trace_config_ctx, params):
        print("Ending request")

    def set_trace_config(self):
        """
        async with aiohttp.ClientSession(
                trace_configs=[trace_config]) as client:
            client.get('http://example.com/some/redirect/')
        async with aiohttp.ClientSession(
                trace_configs=[AuditRequest(), XRay()]) as client:
            client.get('http://example.com/some/redirect/')
        :return:
        """
        # from mylib.traceconfig import AuditRequest
        # from mylib.traceconfig import XRay
        trace_config = aiohttp.TraceConfig()
        trace_config.on_request_start.append(self.on_request_start)
        trace_config.on_request_end.append(self.on_request_end)

    def tpc_conn(self, limit=100, limit_per_host=30,
                 ttl_dns_cache=300, use_dns_cache=False, resolver=None):
        """
        默认情况下，会话对象获得连接器的所有权，此外，一旦关闭会话，连接对象也会关闭连接。
        如果您希望通过不同的会话 实例共享同一连接器，
        则必须 为每个会话实例将 connector_owner参数设置为False。


        :param limit:   要限制同时打开的连接数，您可以将limit 参数传递给连接器：
        如果您明确希望没有限制，请传递0。例如：conn = aiohttp.TCPConnector(limit=0)
        :param limit_per_host:    要限制到同一端点（三重）的同时打开的连接数，可以将limit_per_host 参数传递给connector：(host, port, is_ssl)
        该示例将并行连接的数量限制为30个。
        默认值为0（每个主机没有限制）。
        :param ttl_dns_cache:   要限制到同一端点（三重）的同时打开的连接数，可以将limit_per_host 参数传递给connector：(host, port, is_ssl)
        该示例将并行连接的数量限制为30个。
        默认值为0（每个主机没有限制）。
        :param use_dns_cache:或禁用DNS缓存表的使用，这意味着所有请求最终都会做出DNS解析，如以下示例所示：
        :param resolver:  resolver = AsyncResolver(nameservers=["8.8.8.8", "8.8.4.4"])
        :return:
        """
        connector = aiohttp.TCPConnector(limit=limit, limit_per_host=limit_per_host,
                                         ttl_dns_cache=ttl_dns_cache, use_dns_cache=use_dns_cache,
                                         resolver=resolver)
        return connector

    def unix_conn(self,path):
        conn = aiohttp.UnixConnector(path=path)
        return conn

    def windows_conn(self,path):
        """
        它仅适用于ProactorEventLoop
        :param path:
        :return:
        """
        conn = aiohttp.NamedPipeConnector(path=path)
        return conn

    def get_cookie_jar(self):
        """
        session.get(
        'http://httpbin.org/cookies/set?my_cookie=my_value')
        filtered = session.cookie_jar.filter_cookies(
        'http://httpbin.org')
        assert filtered['my_cookie'].value == 'my_value'
        :return:
        """
        return self.session.cookie_jar

    def url(self, url, encoded=True):
        """
        传递参数将覆盖encoded=True，切勿同时使用这两个选项。
        self.params 为 传递参数
        aiohttp在发送请求之前在内部执行URL 规范化。
        册封编码主机通过部分IDNA编解码器和适用 requoting到路径和查询部分。
        例如URL('http://example.com/путь/%30?a=%31')转换为 URL('http://example.com/%D0%BF%D1%83%D1%82%D1%8C/0?a=1')。
        如果服务器接受精确的表示并且不重新引用URL本身，则有时不希望进行规范化。
        :param url:
        :param encoded: 要禁用规范化，请使用encoded=True参数进行URL构建
        :return:
        """
        return URL(url, encoded=encoded)

    def get(self):
        """
        如果您需要设置自定义ssl参数（例如，使用自己的认证文件），则可以创建一个ssl.SSLContext实例并将其传递给适当的ClientSession方法
        sslcontext = ssl.create_default_context(
            cafile='/path/to/ca-bundle.crt')
        如果需要验证自签名证书，则可以执行与上一个示例相同的操作，但是ssl.SSLContext.load_cert_chain()使用密钥对添加另一个调用 ：
        sslcontext = ssl.create_default_context(
        cafile='/path/to/ca-bundle.crt')
        sslcontext.load_cert_chain('/path/to/client/public/device.pem',
                                   '/path/to/client/private/device.key')
        ssl验证失败时存在显式错误
        aiohttp.ClientConnectorSSLError
        ssl=False aiohttp对HTTPS协议使用严格检查。认证检查可以通过设置适当放宽
        :return:
        """
        pass

    def post(self):
        pass

    def put(self):
        pass

    def delete(self):
        pass

    def head(self):
        pass

    def options(self):
        pass

    def patch(self):
        pass

    def rsp(self):
        """
        print(resp.status)
        print(await resp.text())
        aiohttp自动解码服务器中的内容。您可以为该text()方法指定自定义编码：
        await resp.text(encoding='windows-1251')
        对于非文本请求，您还可以字节形式访问响应主体：
        print(await resp.read())
        br编码解码需要：
        https://github.com/python-hyper/brotlipy
        # 如果JSON解码失败，json()将引发异常。可以为json()调用指定自定义编码和解码器功能。
        print(await resp.json())
        # 上面的方法将整个响应主体读入内存。如果您打算读取大量数据，请考虑使用下面记录的流响应方法。
        # 您可以使用该content 属性。它是aiohttp.StreamReader 该类的一个实例。
        # 在gzip和deflate转移编码自动进行解码为您提供：
        with open(filename, 'wb') as fd:
            while True:
                chunk = await resp.content.read(chunk_size)
                if not chunk:
                    break
                fd.write(chunk)

        print(await resp.headers)
        # 但如果服务器使用非标准编码，则有时需要未转换的数据。
        这些标头的格式不正确RFC 7230的 角度来看，可以通过使用
        print(await resp.raw_headers)
        如果响应包含一些HTTP Cookie，则可以快速访问它们：
        响应Cookie仅包含重定向链中最后一个请求的Set-Cookie标头中的值。
        要在所有重定向请求之间收集cookie，请使用aiohttp.ClientSession对象。
        print(await resp.cookies)
        # 如果请求被重定向，则可以使用history属性查看先前的响应：
        如果未发生重定向或将其allow_redirects设置为False，则历史记录将为空序列。
        print(await resp.history)
        显式地传递期望的类型（在这种情况下，检查将是严格的，没有扩展格式的支持，因此custom/xxx+type将不被接受）：
        await resp.json(content_type='custom/type')。
        完全禁用检查：
        await resp.json(content_type=None)。
        :return:
        """
        pass

    def multipart(self):
        """
        https://docs.aiohttp.org/en/stable/multipart.html#aiohttp-multipart
        :return:
        """
        pass

    def close(self):
        self.session.close()

    def __del__(self):
        self.close()



import pytest

from re_common.baselibrary.utils.mfaker import MFaker


# pytest mfaker_test.py -s
# -s 打印print的结果
def test_faker_dict():
    fake = MFaker()
    dicts = fake.create_data(MFaker.m_pydict, **fake.py_para())
    print(dicts)
    print(len(dicts))
    assert type(dicts) == dict


# pytest mfaker_test.py::test_faker_list -s
# 测试单独的一个test
def test_faker_list():
    fake = MFaker()
    lists = fake.create_data(MFaker.m_pylist, **fake.py_para())
    print(lists)
    print(len(lists))
    assert type(lists) == list










# -*- coding: utf-8 -*-
# @Time    : 2020/4/30 16:04
# @Author  : suhong
# @File    : hyxb.py
# @Software: PyCharm


# 1.海洋学报：英文版

import sys


import requests
from bs4 import BeautifulSoup
# from parsel import Selector
import  parent

hdrs = {"User-Agent":"Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/75.0.3770.100 Safari/537.36",
        "Referer": "http://www.hyxb.org.cn/aosen/ch/reader/issue_list.aspx",
        }

Parent = parent.Parent()

class Hyxb(object):

    def __init__(self):
        self.journal = '海洋学报(英文版)'
        self.base_url = "http://www.hyxb.org.cn/aosen/ch/reader/"
        self.ero = 0
        self.suc = 0
        self.cnt = 0
        self.all_num = 0

    def getpdf(self, years, num, pdfRoot):
        years = str(years)
        num = str(num)
        url  = "http://www.hyxb.org.cn/aosen/ch/reader/issue_list.aspx"
        print(url)
        try:
            data = {
                'year_id': years,
                'quarter_id': num
            }
            r = requests.post(url, headers=hdrs,data=data, timeout=60)
        except:

            print('---------------访问失败--------------')
            return False
        line = 'No.' + str(num)
        print(line)
        soup = BeautifulSoup(r.text, 'lxml')
        all_table = soup.find_all('table', id='table24')
        if len(all_table) == 1:
            print("年份或者期刊错误")
            return False
        for table in all_table:
            for table_2 in table.find_all("table",width='100%'):
                for t in table_2:
                    a_num = len(t.find_all("a"))
                    if a_num == 3:
                        self.all_num += 1
                    if "Package" in t.get_text():
                        self.all_num -= 1
                        continue
                    continue
        title = 1
        for table in all_table:
            for table_2 in table.find_all("table", width='100%'):
                for t in table_2:
                    a_num = len(t.find_all("a"))
                    if a_num == 3:
                        pdfurl = self.base_url + t.find_all('td')[2].find("a").get('href')
                        print('-----------开始下载-------------: %s' % title)
                        if "Package" in t.get_text():
                            p_title = years + '-' + num + '整刊'
                            pdfFilePath = Parent.makedir(self.journal, years, str(num), str(p_title), pdfRoot)
                        else:
                            pdfFilePath = Parent.makedir(self.journal, years, str(num), str(title), pdfRoot)
                            if type(title) == int:
                                title += 1
                        if pdfFilePath == 1:
                            self.suc += 1
                            continue
                        fig = Parent.getPdf(pdfurl, title, pdfFilePath, hdrs)
                        if fig == -1:
                            self.ero += 1
                            line = 'journal: %s, years: %s, issue: %s: %s 下载失败' % (
                                self.journal, years, num, str(title))
                            line = line + '\n'
                            Parent.Record(line, self.journal)
                            # print(line)
                        if fig == 1:
                            print('%s,pdf下载成功' % (title-1))
                            self.suc += 1
                            line = 'journal: %s, years: %s, issue: %s。共有文章 %d；下载PDF %d；失败 %d' % (
                                self.journal, years, num, self.all_num+1, self.suc, self.ero)
                            line = line + '\n'
                            Parent.Record(line, self.journal)
        Parent.Record("\n", self.journal)
        return True
# if __name__ == '__main__':
#     h = Hyxb()
#     h.getpdf("2020",'4','./')


from re_common.baselibrary.utils.baselist import BaseList


class JavaCodeDeal(object):

    def __init__(self):
        self.strings = """
        """
        self.lists = []
        self.baselist = BaseList()
        self.keylist = []

    def set_strings(self, strings):
        """
        设置源数据
        :param strings:
        :return:
        """
        self.strings = strings

    def strings_split(self, split="\n"):
        """
        根据条件切割为单独的数据
        :param split:
        :return:
        """
        self.lists = self.strings.split(split)

    def list_deal(self):
        """
        去除空字符串，空数据，以及字符串前后空格
        :return:
        """
        self.lists = self.baselist.remove_null(self.lists)
        self.lists = self.baselist.remove_blank_space(self.lists)
        self.lists = self.baselist.clean_space_first_end(self.lists)

    def get_key(self):
        """
        获取等号前的变量
        :return:
        """
        for item in self.lists:
            key = item.split("=")[0].replace("String", "").strip()
            self.keylist.append(key)

    def set_outmap(self):
        """
        生成outMap的put代码
        :return:
        """
        for item in self.lists:
            key = item.split("=")[0].replace("String", "").strip()
            print('outMap.put("{}", {});'.format(key, key))

    def set_sql_deal(self):
        """
        生成mysql处理代码
        :return:
        """
        for item in self.lists:
            key = item.split("=")[0].replace("String", "").strip()
            print("{} = {}.replace('\\0', ' ').replace(\"'\", \"''\").trim();".format(key, key))

    def create_sql(self):
        strinsgs = "String sql = \"INSERT INTO modify_title_info_zt("
        for key in self.keylist:
            strinsgs += "[{}],".format(key)
        strinsgs = strinsgs[:-1]
        strinsgs += ")\";"
        print(strinsgs)

    def create_s(self):
        Strings = "sql += \" VALUES ("
        for key in self.keylist:
            Strings += "'%s', "
        Strings = Strings[:-2]
        Strings += ")\";"
        print(Strings)

    def create_value(self):
        Strings = "sql = String.format(sql,"
        for key in self.keylist:
            Strings += key + ","
        Strings = Strings[:-1]
        Strings += ");"
        print(Strings)

    def create_java_sql(self):
        self.create_sql()
        self.create_s()
        self.create_value()


if __name__ == "__main__":
    strings = """
     String rawid = outMap.getOrDefault("rawid", "");
            String lngid = outMap.getOrDefault("lngid", "");
            gch = gch;
            String title = outMap.getOrDefault("rawid", "");
            String title_alternative = outMap.getOrDefault("title_alt", "");
            String identifier_issn = outMap.getOrDefault("issn", "");
            String identifier_cnno = outMap.getOrDefault("cnno", "");
            String creator = outMap.getOrDefault("author", "");
            String creator_en = outMap.getOrDefault("author_raw", "");
            String creator_institution = outMap.getOrDefault("organ", "");
            String source = outMap.getOrDefault("journal_name", "");
            String source_en = outMap.getOrDefault("journal_name_alt", "");
            String date = StringHelper.getYear(pub_year);
            String volume = outMap.getOrDefault("vol", "");
            String issue = outMap.getOrDefault("num", "");
            String description = outMap.getOrDefault("abstract", "");
            String description_en = outMap.getOrDefault("abstract_alt", "");
            String description_fund = outMap.getOrDefault("fund", "");
            String description_core = "";
            String subject = outMap.getOrDefault("keyword", "");
            String subject_en = outMap.getOrDefault("keyword_alt", "");
            String begin_page = outMap.getOrDefault("begin_page", "");
            String end_page = outMap.getOrDefault("end_page", "");
            String page = format_page(begin_page, end_page);
            String subject_clc = clc_no.replace(" ", ";").replace(",", ";");//分类号，将空格，逗号替换为分号
            String date_created = format_data_create(publishdate, pub_year);//日期，不是4位的解析出四位数字，如果是空和1900开头，改成data+0000
            String identifier_doi = outMap.getOrDefault("doi", "");
            String country = "CN";
            String language = "ZH";
            String provider = "cnkijournal";
            String owner = "cqu";
            String type = "3";
            String medium = "2";
            batch = batch;
            String provider_url = "cnkijournal@http://kns.cnki.net/KCMS/detail/detail.aspx?dbcode=CJFD&filename=" + rawid;
            String provider_id = "cnkijournal@" + rawid;
    """
    j = JavaCodeDeal()
    j.set_strings(strings)
    j.strings_split()
    j.list_deal()
    j.get_key()
    j.create_java_sql()



from re_common.baselibrary.utils.basepika import BasePika


class UseMq(object):

    def __init__(self, queue, qos=1):
        self.queue = queue
        self.basepika = BasePika()
        self.basepika.set_default()
        self.basepika.connect()
        self.basepika.create_channel()
        self.basepika.queue_declare(queue=queue, durable=True)
        self.basepika.basic_qos(qos)
        self.properties = self.basepika.get_properties()

    def get_mq(self):
        self.basepika.set_get_msg_callback(routing_key=self.queue, callback=self.callback, auto_ack=False)
        self.basepika.start_get_msg()

    def callback(self, ch, method, properties, body):
        # print(type(body))
        # print(" [x] Received %r" % body)
        # body = body.decode()
        self.callback2(ch, method, properties, body)
        if self.basepika.auto_ack is False:
            self.basepika.basic_ack(ch, method)

    def callback2(self, ch, method, properties, body):
        pass

    def send_mq(self, body, num=100):
        if self.basepika.get_queue_size(self.queue) < num:
            self.basepika.easy_send_msg(routing_key=self.queue,
                                        body=body,
                                        properties=self.properties)
            return True
        else:
            return False



# -*- coding: utf-8 -*-
# @Time    : 2019/7/4 16:20
# @Author  : suhong
# @FileName: stxb.py
# @Software: PyCharm
import sys
import time

import requests
import  parent
from urllib import parse
from bs4 import BeautifulSoup
from lxml import etree

data = {}
hdrs = {"User-Agent":"Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/75.0.3770.100 Safari/537.36",
        "Referer" :"http://www.ecologica.cn",

}
Parent = parent.Parent()
class stxb(object):


    def __init__(self):
        self.url = 'http://www.ecologica.cn/stxb/ch/reader/issue_list.aspx?'
        self.ero = 0
        self.suc = 0
        self.cnt = 0
        self.title_c = 1
        self.journal = '生态学报'

    def getpdf(self,years, num,pdfRoot):
        years = str(years)
        num = str(num)
        url = 'http://www.ecologica.cn/stxb/ch/reader/issue_list.aspx?year_id=%s&quarter_id=%s'%(years,num)


        try:
            r = requests.get(url, headers=hdrs,  timeout =60)

            time.sleep(5)
        except:
            print('---------------访问失败------------')
            return False

        html = etree.HTML(r.content.decode())
        all_table = html.xpath("//table[@id='table24']")
        for tables in all_table[1:]:

            # 获取当前节点下所有的td width="430"
            all_td = tables.xpath("//td[@width='430']")
            self.cnt = len(all_td)

            for one in all_td:
                title =""
                id=""
                url=""
                # print(one.xpath("./a/text()"))
                # print(  one.xpath("normalize-space(./a/text())"))
                # print( one.xpath("./a/@href")[0].split("&")[0].split("=")[-1])
                try:
                    title = str(self.title_c)
                    # title = one.xpath("normalize-space(./a/text())")
                    print("pdf下载title:%s" % title)
                    id = one.xpath("./a/@href")[0].split("&")[0].split("=")[-1]
                    print("pdf下载id:%s" % id)
                    url ="http://www.ecologica.cn/stxb/ch/reader/create_pdf.aspx?file_no={}&quater_id=2&falg=1".format(id)

                    print("pdf下载链接：%s" % url)
                except Exception as e:
                    print("--------页面存在异常格式:%s-%s,错误原因:%s" % (years, num,str(e)))
                    sys.exit(-1)

                print('--------开始下载---------: %s' % title)

                pdfFilePath =Parent.makedir(self.journal, years, str(num),title,pdfRoot)
                if pdfFilePath == 1:
                    self.suc += 1
                    continue
                fig = Parent.getPdf(url, title, pdfFilePath,hdrs)
                if fig == -1:
                    self.ero += 1
                    line = 'journal: %s, years: %s, issue: %s: %s 下载失败' % (
                    self.journal, years, num, title.strip())
                    line = line + '\n'
                    Parent.Record(line,self.journal)
                if fig == 1:
                    print('%s,pdf下载成功' % title)
                    self.suc += 1
                    self.title_c += 1

                    line = 'journal: %s, years: %s, issue: %s。共有文章 %d；下载PDF %d；失败 %d' % (
                    self.journal, years, num, self.cnt, self.suc, self.ero)
                    line = line + '\n'
                    Parent.Record(line,self.journal)

            Parent.Record("\n", self.journal)
            return True


if __name__ == "__main__":
    work =stxb()
    work.getpdf("2015","3","./")


import pymysql as pymysql

from re_common.baselibrary.database.moudle import SqlMoudle
from re_common.baselibrary.database.sql_factory import SqlFactory


class Mysql(SqlFactory):

    @property
    def db(self):
        # 就是conn
        if not self.is_ping():
            assert self._mysqlmoudle, AttributeError("MySQL model不存在 无法重新连接 请设置moudle")
            self.reConnect()
        return self._db  # type: pymysql.cursors.Cursor

    @db.setter
    def db(self, value):
        self._db = value

    @property
    def cursor(self):
        if self._cursor.connection:
            return self._cursor  # type: pymysql.cursors.Cursor
        else:
            self._cursor = self._db.cursor()
            return self._cursor

    @cursor.setter
    def cursor(self, value):
        self._cursor = value

    @property
    def mysqlmoudle(self):
        return self._mysqlmoudle  # type: SqlMoudle

    @mysqlmoudle.setter
    def mysqlmoudle(self, moudle):
        self._mysqlmoudle = moudle

    def get_new_cursor(self):
        """
        获取一个新的游标
        :return:
        """
        # 检查db的存在及是否断掉 不知道为什么 不可以加括号 但编辑器会警告
        return self._db.cursor()

    def link(self, mysqlmoudle: SqlMoudle):
        """
        连接数据库
        :param mysqlmoudle:
        :return:
        """
        self._mysqlmoudle = mysqlmoudle
        # 返回连接对象
        self._db = pymysql.connect(**mysqlmoudle.to_dict())
        self._cursor = self._db.cursor()

        return self

    def reConnect(self):
        """
        重新连接数据库
        :return:
        """
        try:
            self._db.ping()
            self._db.close()
        except:
            self.link(self._mysqlmoudle)
        self.link(self._mysqlmoudle)

    def is_ping(self):
        if self._db:
            try:
                return self._db.ping()
            except:
                self.link(self._mysqlmoudle)
        else:
            raise AttributeError("数据库连接对象不存在，请调用reConnect重新连接")

    def commit(self):
        """
        事务提交
        :return:
        """
        # 对象判断
        assert isinstance(self._db, pymysql.connections.Connection)
        self._db.commit()

    def execute(self, sql):
        """
        执行sql语句
        :param sql:
        :return:
        """
        try:
            assert isinstance(self._cursor, pymysql.cursors.Cursor)
            # 游标是否被关闭,执行该函数保证有游标
            self._cursor.execute(sql)
            self.commit()
        except Exception as e:
            raise e

    def fetchall(self):
        """
        select 后调用获取全部返回结果
        :return:
        """
        result = self._cursor.fetchall()
        return result

    def close(self):
        assert isinstance(self._db, pymysql.connections.Connection)
        self._db.close()

    def rollback(self):
        self.db.rollback()

    def escape(self, strings):
        return self.db.escape(strings)

    def __repr__(self):
        return 'Mysql().link(%s)' % self._mysqlmoudle

    def __str__(self):
        return "Mysql %s" % self._mysqlmoudle

    def __del__(self):
        self.close()



import os
import shutil
import warnings

from re_common.baselibrary.utils.basefile import BaseFile


class BaseDir(object):

    @classmethod
    def get_dir_all_files(cls, path):
        """
        返回所有的文件 不会返回目录 目的是得到一个文件列表
        # 这里注意topdown参数。
        # topdown决定遍历的顺序，
        # 如果topdown为True，则先列举top下的目录，然后是目录的目录，依次类推；
        # 反之，则先递归列举出最深层的子目录，然后是其兄弟目录，然后父目录。
        # 我们需要先修改深层的子目录
        :param path:  给定一个路径 如 C:/Users/XD/Pictures/所有视频/
        :return:
        """
        # 遍历子目录
        assert cls.is_dir(path), NotADirectoryError(path + " not is a dir please check")
        for path, dirNames, fileNames in os.walk(path, topdown=True):
            # 获取当前目录的所有文件
            for fileName in fileNames:
                yield os.path.join(path, fileName)

    @classmethod
    def is_dir_exists(cls, pathdir):
        """
        目录是否存在
        :param sPth:
        :return:
        """
        return os.path.exists(pathdir)

    @classmethod
    def create_dir(cls, pathdir):
        """
        创建目录 该方法可以创建多级目录
        os.mkdir(pathdir)是创建一级目录 该目录的上级目录必须存在
        :param pathdir:
        :return:
        """
        if not cls.is_dir_exists(pathdir):
            os.makedirs(pathdir)

    @classmethod
    def get_upper_dir(cls, curPath: str, num: int) -> str:
        """
        获取上几层的目录
        :param curPath:
        :param num:
        :return:
        """
        # 如果不是个整数
        if not isinstance(num, int) or num > 0:
            warnings.warn("this not int or num > 0 , The path is not processed, return curPath")
            return curPath
        pathlist = curPath.split(os.sep)
        if abs(num) > (len(pathlist) - 1):
            warnings.warn("this path length < num ,will return root dir ")
            num = (len(pathlist) - 1) * (-1)
        pathlist = pathlist[:num]
        TopPath = os.sep.join(pathlist)
        return TopPath

    @classmethod
    def get_file_dir_absolute(cls, file):
        """
        请传入 __file__ 作为参数
        获取一个文件的绝对路径
        :param __file__:
        :return:
        """
        curPath = os.path.dirname(os.path.abspath(file))
        return curPath

    @classmethod
    def is_dir(cls, spath):
        if os.path.isdir(spath):
            return True
        return False

    @classmethod
    def get_dir_size(cls, dirpath):
        """
        1eb = 1024pb
        1pb = 1024tb
        1tb = 1024gb
        1gb = 1024mb
        1mb = 1024kb
        1kb = 1024b
        上面的b是Byte
        1Byte = 8bit
        :param dirpath:
        :return: 这里返回的是Byte大小
        """
        size = 0
        for filepath in cls.get_dir_all_files(dirpath):
            size += sum(BaseFile.get_file_size(filepath))
        return size

    @classmethod
    def copy_file_to_dir(cls, filePath, tarDirPath):
        """

        :param filePath:  文件路径
        :param tarDirPath:  目录或文件路径
        :return:
        """
        assert BaseFile.is_file_exists(filePath), FileNotFoundError("需要copy的文件不存在")
        assert BaseFile.is_file(filePath), FileNotFoundError("需要copy的不是一个文件")
        cls.create_dir(tarDirPath)
        shutil.copy(filePath, tarDirPath)

    @classmethod
    def copy_dir_to_dir(cls, oldDir, newDir, moudle=0):
        """
        olddir和newdir都只能是目录，且newdir必须不存在
        :param oldDir:
        :param newDir:
        :return:
        """
        assert cls.is_dir(oldDir), IsADirectoryError(oldDir + "目录不存在")
        if moudle == 0:
            # 该模式下新目录必须不存在
            assert not cls.is_dir(newDir), IsADirectoryError(newDir + "目录存在，不能使用该模式,推荐使用模式2")
            shutil.copytree(oldDir, newDir)
        elif moudle == 1:
            # 该模式下如果新目录存在先删除新目录
            if cls.is_dir(newDir):
                cls.delete_dir(newDir)
            shutil.copytree(oldDir, newDir)
        elif moudle == 3:
            # 该模式下通过替换进行copy
            pass
        elif moudle == 4:
            # 该模式会跳过已存在的文件 只将新文件放入其中
            pass

    @classmethod
    def delete_dir(cls, dirpath):
        """
         os.rmdir(dirpath) 只能删除空目录
        os.removedirs(path)  递归的删除目录
        :param dirpath:
        :return:
        """
        shutil.rmtree(dirpath)

    @classmethod
    def replace_dir_special_string(cls, strings):
        """
        windows 目录非法字符
        :param strings:
        :return:
        """
        strings1 = strings.replace("/", "").replace("\\", "") \
            .replace(":", "").replace("*", "").replace("\"", "") \
            .replace("<", "").replace(">", "").replace("|", "") \
            .replace("?", "")
        return strings1

    @classmethod
    def remove_file_suf(cls, dirpath, suf):
        for file in cls.get_dir_all_files(dirpath):
            if file.endswith(suf):
                BaseFile.remove_file(file)

    @classmethod
    def is_null_dir(cls, dirpath):
        """
        是否为空目录
        :param dirpath:
        :return:
        """
        if not os.listdir(dirpath):
            return True
        return False

    @classmethod
    def get_dir_file_num(cls, dirpath):
        return len([name for name in os.listdir(dirpath) if os.path.isfile(os.path.join(dirpath, name))])

    @classmethod
    def get_dir_all_file_num(cls, dirpath, ext=None):
        """
        获取目录下所有文件个数，包含目录下目录的个数
        :return:
        """
        count = 0
        for path in cls.get_dir_all_files(dirpath):
            if BaseFile.is_file(path):
                if ext is not None:
                    if path.endswith(ext):
                        count += 1
            elif cls.is_dir(path):
                count = count + cls.get_dir_all_file_num(path, ext)
        return count

    @classmethod
    def get_dir_all_file_size(cls, dirpath, ext=None):
        """
        获取目录下所有文件的总大小
        :return:
        """
        count = 0
        for path in cls.get_dir_all_files(dirpath):
            if BaseFile.is_file(path):
                if ext is not None:
                    if path.endswith(ext):
                        count = count + BaseFile.get_file_size(path)
            elif cls.is_dir(path):
                count = count + cls.get_dir_all_file_num(path, ext)
        return count



# -*- coding: utf-8 -*-



"""
CREATE YOUR DEFAULT_CONFIG !

Some configuration:
        CONCURRENT_REQUESTS     并发数量
        RETRIES                 重试次数
        DOWNLOAD_DELAY          下载延时
        RETRY_DELAY             重试延时
        DOWNLOAD_TIMEOUT        超时限制
        USER_AGENT              用户代理
        LOG_FILE                日志路径
        LOG_LEVEL               日志等级
"""


CONCURRENT_REQUESTS = 20





MIDDLEWARE = [
'middlewares.middleware',
            ]


# PIPELINES = []




DEFAULT_REQUEST_CONFIG = {
    "RETRIES": 0,
    "DOWNLOAD_DELAY": 0,
    "RETRY_DELAY": 0,
    "DOWNLOAD_TIMEOUT": 10,
}


# '''生成日志文件'''
# LOG_FILE = './asyncpy.log'
# LOG_LEVEL = 'DEBUG'



# CLOSESPIDER_TIMEOUT = 10




USER_AGENT = "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/60.0.3100.0 Safari/537.36"


import abc
import json
import json
import sys
import traceback

from re_common.baselibrary.utils.basedir import BaseDir
from re_common.baselibrary.utils.basefile import BaseFile
from re_common.baselibrary.utils.basetime import BaseTime
from re_common.facade.loggerfacade import get_streamlogger

"""
json 格式如下
{
“step1”：{

“isFinish”：False
}
}
"""


# 执行框架图
class GlobalControl(object):
    def __init__(self, file):
        """

        :param file: 传入__file__
        :param loggerdir:
        """
        self.insert(file)
        self.jsonfile = BaseFile.get_new_path(self.curPath, "run.json")
        self.logger = get_streamlogger()
        # 　False 表示更新没有完成,接着更新　ture 表示新一轮更新已经结束
        self.is_restart = False
        self.nowstep = ""
        self.key = ""
        self.readjson()
        self.basetime = BaseTime()
        self.step_dicts = {}

    @abc.abstractmethod
    def imports(self, dicts=None):
        # raise NotImplementedError python2的方法
        pass

    @abc.abstractmethod
    def set_step_dict(self, dicts):
        pass

    def insert(self, file):
        """
        :param file:  __file__
        :return:
        """
        self.curPath = BaseDir.get_file_dir_absolute(file)
        sys.path.insert(0, self.curPath)  # 插入

    def writejson(self):
        jsonstrings = json.dumps(self.jsondicts, ensure_ascii=False)
        BaseFile.single_write_file(jsonstrings, BaseFile.get_new_path(self.curPath, "run.json"))

    def readjson(self):
        if BaseFile.is_file_exists(self.jsonfile):
            self.logger.info("读取json 文件{}".format(str(self.jsonfile)))
            jsonstrings = BaseFile.single_read_file(self.jsonfile)
            self.logger.info("json　文件内容为：{}".format(jsonstrings))
            if jsonstrings:
                self.jsondicts = json.loads(jsonstrings)
            else:
                self.jsondicts = {}
        else:
            self.logger.info("json　文件不存在,创建空的dicts")
            self.jsondicts = {}

    def check(self):
        if "end" in self.jsondicts.keys() and self.jsondicts["end"]["status"] is True:
            self.logger.info("表示上次程序运行结束，本次是新的更新而不是上次发生意外而进行的继续更新")
            self.is_restart = True
        else:
            self.logger.info("表示更新没有完成")
            self.is_restart = False

    def init(self):
        if not self.jsondicts:
            self.logger.info("新一轮更新开始,对json 进行初始化")
            # 清空字典重新开始
            self.jsondicts = {}
            self.jsondicts["start"] = {}
            self.jsondicts["start"]["time"] = self.basetime.get_beijin_date_strins(format="%Y-%m-%d %H:%M:%S")
            self.jsondicts["start"]["isFinish"] = True

    def start_hook(self, key):
        pass

    @abc.abstractmethod
    def step_fun(self):
        """
        调用方法的字符串
        :return:
        """
        pass

    def start(self):
        """
        该步骤获取应该运行哪个代码
        :return:
        """
        thiskey = ""
        for key in self.jsondicts.keys():
            thiskey = key
            if not self.jsondicts[key]["isFinish"]:
                self.key = key
                self.nowstep = self.step_fun()

                return True
            # 如果要对某一个步骤进行特殊处理,可以在该函数返回True
            if self.start_hook(key):
                return True
        if thiskey == "start":
            self.key = "1"
            self.nowstep = self.step_fun()
            return True

        self.key = str(int(thiskey) + 1)
        self.nowstep = self.step_fun()
        return True

    def run(self, strings):
        """
        该步骤运行代码
        :return:
        """
        try:
            if strings == "end":
                return True
            result = eval(strings)
        except:
            self.logger.info(traceback.format_exc())
            return False
        return result

    def writetrue(self, errmessage="", dicts=None):
        self.jsondicts[self.key] = {}
        self.jsondicts[self.key]["errmessage"] = errmessage
        self.jsondicts[self.key]["isFinish"] = True
        self.jsondicts[self.key]["time"] = self.basetime.get_beijin_date_strins(format="%Y-%m-%d %H:%M:%S")
        if dicts is not None:
            self.jsondicts[self.key]["msg"] = dicts
        self.writejson()

    def writefalse(self, e, dicts=None):
        self.jsondicts[self.key] = {}
        self.jsondicts[self.key]["errmessage"] = str(e)
        self.jsondicts[self.key]["isFinish"] = False
        self.jsondicts[self.key]["time"] = self.basetime.get_beijin_date_strins(format="%Y-%m-%d %H:%M:%S")
        if dicts is not None:
            self.jsondicts[self.key]["msg"] = dicts
        self.writejson()

    @abc.abstractmethod
    def one_check(self):
        """
        该步骤检查代码是否运行完成
        :return:
        """
        pass

    def end(self):
        self.jsondicts["end"] = {}
        self.jsondicts["end"]["status"] = True
        self.jsondicts["end"]["time"] = self.basetime.get_beijin_date_strins(format="%Y-%m-%d %H:%M:%S")
        self.writejson()

    def work(self):
        """
        该函数控制整个流程
        :return:
        """
        self.logger.info("start work.....")
        self.check()
        if self.is_restart:
            return True
        self.init()
        while True:
            result1 = self.start()
            self.logger.info("开始第" + self.key + "步")
            result2 = self.run(self.nowstep)
            result3 = self.one_check()
            if result1 and result2 and result3:
                break
        self.end()
